@@ -1,12 +1,12 @@
 /***********************************************************/
 /*                                                         */
 /*   BFONT.c v. 1.0.3 - Billi Font Library by Diego Billi  */
+/*   Heavily modified for FreedroidRPG needs over years    */
 /*                                                         */
 /***********************************************************/
 
 #define _bfont_c
 
-// #include "BFont.h"
 #include "system.h"
 #include "defs.h"
 #include "struct.h"
@@ -16,98 +16,93 @@
 
 
 /* Current font */
-BFont_Info *CurrentFont;
+static BFont_Info *CurrentFont;
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
-void
+ */
+static void
 InitFont (BFont_Info * Font)
 {
-  unsigned int x = 0, i = 0;
-  Uint32 sentry;
-  SDL_Surface* tmp_char1;
+    unsigned int x = 0, i = 0;
+    Uint32 sentry;
+    SDL_Surface* tmp_char1;
 
-  Font->h = Font->Surface->h;
+    Font->h = Font->Surface->h;
 
-  i = '!';
-  // sentry = FdGetPixel ( Font -> Surface, 0, 0);
-  sentry = SDL_MapRGB(Font->Surface->format, 255, 0, 255); 
+    i = '!';
+    sentry = SDL_MapRGB(Font->Surface->format, 255, 0, 255); 
 
-  if (Font->Surface == NULL)
-    {
-      fprintf (stderr, "BFont: The font has not been loaded!\n");
-      exit (1);
-    }
-  if (SDL_MUSTLOCK (Font->Surface))
-    SDL_LockSurface (Font->Surface);
-  x = 0;
-  while (x < (Font->Surface->w - 1) && i < MAX_CHARS_IN_FONT)
-    {
-      if (FdGetPixel (Font->Surface, x, 0) != sentry)
+    if (Font->Surface == NULL)
 	{
-	  Font->Chars[i].x = x;
-	  Font->Chars[i].y = 1;
-	  Font->Chars[i].h = Font->Surface->h;
-          while( x < (Font->Surface->w)){
-               if(FdGetPixel (Font->Surface, x, 0) == sentry)
-                   break;
-               x++;
-          }
-	  Font->Chars[i].w = (x - Font->Chars[i].x);
-
-	  //--------------------
-	  // Now we make a copy for later reference when we do OpenGL based
-	  // output of this character.
-	  //
-	  Font->number_of_chars = i + 1;
-	  tmp_char1 = SDL_CreateRGBSurface( 0 , CharWidth ( Font , i ) , FontHeight (Font) -1 , 32, 
-					    //0xFF000000 , 0x00FF0000  , 0x0000FF00 , 0x000000FF );
-					    0x000000FF , 0x0000FF00  , 0x00FF0000 , 0xFF000000 );
-
-	  SDL_SetAlpha( Font->Surface , 0 , 255 );
-	  SDL_SetColorKey( Font->Surface , 0 , 0 );
-	  Font -> char_iso_image [ i ] . surface = SDL_DisplayFormatAlpha ( tmp_char1 ) ;
-	  Font -> char_iso_image [ i ] . texture_has_been_created = FALSE ;
-	  our_SDL_blit_surface_wrapper ( Font->Surface, & ( Font -> Chars [ i ] ) , 
-					 Font -> char_iso_image [ i ] . surface , NULL );
-	  SDL_SetAlpha( Font -> char_iso_image [ i ] . surface , SDL_SRCALPHA , SDL_ALPHA_OPAQUE );
-	  SDL_SetColorKey ( Font -> char_iso_image [ i ] . surface , 0 , 0 );
-
-	  flip_image_horizontally ( Font -> char_iso_image [ i ] . surface ) ;
-
-	  //--------------------
-	  // Now we can go on to the next char
-	  //
-	  i++;
-	}
-      else
-	{
-	  x++;
+	fprintf (stderr, "BFont: The font has not been loaded!\n");
+	exit (1);
 	}
 
-    }
-  Font->Chars[' '].x = 0;
-  Font->Chars[' '].y = 0;
-  Font->Chars[' '].h = Font->Surface->h;
-  Font->Chars[' '].w = Font->Chars['!'].w;
-  Font->number_of_chars = i;
+    if (SDL_MUSTLOCK (Font->Surface))
+	SDL_LockSurface (Font->Surface);
+
+    x = 0;
+    while (x < (Font->Surface->w - 1) && i < MAX_CHARS_IN_FONT)
+	{	
+	if (FdGetPixel (Font->Surface, x, 0) != sentry)
+	    {
+	    Font->Chars[i].x = x;
+	    Font->Chars[i].y = 1;
+	    Font->Chars[i].h = Font->Surface->h;
+	    while( x < (Font->Surface->w))
+		{
+		if(FdGetPixel (Font->Surface, x, 0) == sentry)
+		    break;
+		x++;
+	        }
+	    Font->Chars[i].w = (x - Font->Chars[i].x);
+
+	    Font->number_of_chars = i + 1;
+
+	    tmp_char1 = SDL_CreateRGBSurface( 0 , CharWidth ( Font , i ) , FontHeight (Font) -1 , 32, rmask, gmask, bmask, amask );
+	    Font -> char_iso_image [ i ] . surface = SDL_DisplayFormatAlpha ( tmp_char1 ) ;
+	    Font -> char_iso_image [ i ] . texture_has_been_created = FALSE ;
+
+	    our_SDL_blit_surface_wrapper ( Font->Surface, & ( Font -> Chars [ i ] ) , Font -> char_iso_image [ i ] . surface , NULL );
+	    SDL_SetAlpha( Font -> char_iso_image [ i ] . surface , SDL_SRCALPHA , SDL_ALPHA_OPAQUE );
+	    SDL_SetColorKey ( Font -> char_iso_image [ i ] . surface , 0 , 0 );
+
+	    flip_image_vertically ( Font -> char_iso_image [ i ] . surface ) ;
+
+	    SDL_FreeSurface ( tmp_char1 );
+	    //--------------------
+	    // Now we can go on to the next char
+	    //
+	    i++;
+	    }
+	else
+	    {
+	    x++;
+	    }
+
+	}
+    Font->Chars[' '].x = 0;
+    Font->Chars[' '].y = 0;
+    Font->Chars[' '].h = Font->Surface->h;
+    Font->Chars[' '].w = Font->Chars['!'].w;
+    Font->number_of_chars = i;
 
 #ifdef HAVE_LIBGL
-if ( use_open_gl ) 
-  Font->list_base = glGenLists(i);
+    if ( use_open_gl )
+	Font->list_base = glGenLists(i);
 #endif
-  if (SDL_MUSTLOCK (Font->Surface))
-    SDL_UnlockSurface (Font->Surface);
+    if (SDL_MUSTLOCK (Font->Surface))
+	SDL_UnlockSurface (Font->Surface);
 
-  SDL_SetColorKey ( Font->Surface, 0 , 0 );
-  SDL_SetAlpha( Font -> Surface , SDL_SRCALPHA , SDL_ALPHA_OPAQUE );
+    SDL_SetColorKey ( Font->Surface, 0 , 0 );
+    SDL_SetAlpha( Font -> Surface , SDL_SRCALPHA , SDL_ALPHA_OPAQUE );
 }; // void InitFont (BFont_Info * Font)
 
-/* ----------------------------------------------------------------------
+/**
  * Load the font and stores it in the BFont_Info structure 
- * ---------------------------------------------------------------------- */
+ */
 BFont_Info *
 LoadFont (const char *filename)
 {
@@ -120,15 +115,6 @@ LoadFont (const char *filename)
       Font = (BFont_Info *) MyMalloc (sizeof (BFont_Info));
       if (Font != NULL)
 	{
-	  //--------------------
-	  // Here we cannot use our image loading wrapper, cause that one
-	  // would also flip the image horizontally, which would destroy the
-	  // top character info!
-	  // 
-	  // Too bad!
-	  //
-	  // tmp = (SDL_Surface *) our_IMG_load_wrapper (filename);
-	  //
 	  tmp = (SDL_Surface *) IMG_Load (filename);
 
 	  if (tmp != NULL)
@@ -142,6 +128,9 @@ LoadFont (const char *filename)
 		  Font->Chars[x].h = 0;
 		  Font->Chars[x].w = 0;
 		}
+	      SDL_SetAlpha( Font->Surface , 0 , 255 );
+	      SDL_SetColorKey( Font->Surface , 0 , 0 );
+
 	      /* Init the font */
 	      InitFont (Font);
 	      /* Set the font as the current font */
@@ -160,10 +149,10 @@ LoadFont (const char *filename)
   return Font;
 }
 
-/* ---------------------------------------------------------------------- 
+/** 
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 FreeFont (BFont_Info * Font)
 {
@@ -171,132 +160,33 @@ FreeFont (BFont_Info * Font)
   free (Font);
 }; // void FreeFont (BFont_Info * Font)
 
-/* ----------------------------------------------------------------------
- *
- *
- * ---------------------------------------------------------------------- */
-BFont_Info *
-SetFontColor (BFont_Info * Font, Uint8 r, Uint8 g, Uint8 b)
-{
-  int x, y;
-
-  BFont_Info *newfont;
-  SDL_Surface *surface = NULL;
-
-  Uint32 pixel;
-  Uint8 old_r, old_g, old_b;
-  Uint8 new_r, new_g, new_b;
-  Uint32 color_key;
-
-  newfont = (BFont_Info *) malloc (sizeof (BFont_Info));
-  if (newfont != NULL)
-    {
-
-      newfont->h = Font->h;
-
-      for (x = 0; x < 256; x++)
-	{
-	  newfont->Chars[x].x = Font->Chars[x].x;
-	  newfont->Chars[x].y = Font->Chars[x].y;
-	  newfont->Chars[x].h = Font->Chars[x].h;
-	  newfont->Chars[x].w = Font->Chars[x].w;
-	}
-
-      surface =
-	SDL_ConvertSurface (Font->Surface, Font->Surface->format,
-			    Font->Surface->flags);
-      if (surface != NULL)
-	{
-
-	  if (SDL_MUSTLOCK (surface))
-	    SDL_LockSurface (surface);
-	  if (SDL_MUSTLOCK (Font->Surface))
-	    SDL_LockSurface (Font->Surface);
-
-	  color_key = FdGetPixel (surface, 0, surface->h - 1);
-
-	  //	  printf ("looking...\n");
-	  for (x = 0; x < Font->Surface->w; x++)
-	    {
-	      for (y = 0; y < Font->Surface->h; y++)
-		{
-		  old_r = old_g = old_b = 0;
-		  pixel = FdGetPixel (Font->Surface, x, y);
-
-		  if (pixel != color_key)
-		    {
-		      SDL_GetRGB (pixel, surface->format, &old_r, &old_g,
-				  &old_b);
-
-		      new_r = (Uint8) ((old_r * r) / 255);
-		      new_g = (Uint8) ((old_g * g) / 255);
-		      new_b = (Uint8) ((old_b * b) / 255);
-
-		      pixel =
-			SDL_MapRGB (surface->format, new_r, new_g, new_b);
-		      PutPixel (surface, x, y, pixel);
-		    }
-		}
-	    }
-	  //	  printf ("unlooking...\n");
-	  if (SDL_MUSTLOCK (surface))
-	    SDL_UnlockSurface (surface);
-	  if (SDL_MUSTLOCK (Font->Surface))
-	    SDL_UnlockSurface (Font->Surface);
-
-	  SDL_SetColorKey (surface, SDL_SRCCOLORKEY, color_key);
-	}
-
-      newfont->Surface = surface;
-    }
-  return newfont;
-}; // BFont_Info* SetFontColor (BFont_Info * Font, Uint8 r, Uint8 g, Uint8 b)
-
-/* ----------------------------------------------------------------------
+/**
  * Set the current font 
- * ---------------------------------------------------------------------- */
+ */
 void
 SetCurrentFont (BFont_Info * Font)
 {
-  if ( Font == NULL )
-    {
-      //--------------------
-      // If we've received a NULL pointer for the font to be set, we make
-      // a big noise!!!
-      //
-      DebugPrintf ( -1000 , "\nNull pointer received in SetCurrentFont!  -->  delibarately causing SEGFAULT...\n\n" );
-      raise ( SIGSEGV );
-    }
   CurrentFont = Font;
 }; // void SetCurrentFont (BFont_Info * Font)
 
-/* ----------------------------------------------------------------------
+/**
  * Returns the pointer to the current font strucure in use 
- * ---------------------------------------------------------------------- */
+ */
 BFont_Info *
 GetCurrentFont (void)
 {
   return CurrentFont;
 }; // BFont_Info * GetCurrentFont (void)
 
-/* ----------------------------------------------------------------------
+/**
  * Return the font height 
- * ---------------------------------------------------------------------- */
+ */
 int
 FontHeight (BFont_Info * Font)
 {
   return (Font->h);
 }; // int FontHeight (BFont_Info * Font)
 
-/* ----------------------------------------------------------------------
- *
- *
- * ---------------------------------------------------------------------- */
-void
-SetFontHeight (BFont_Info * Font, int height)
-{
-  Font->h = height;
-}; // void SetFontHeight (BFont_Info * Font, int height)
 
 /* Return the width of the "c" character */
 int
@@ -307,18 +197,9 @@ CharWidth (BFont_Info * Font, unsigned char c)
   return Font->Chars[c].w;
 }
 
-/* ----------------------------------------------------------------------
- * Puts a single char on the surface 
- * ---------------------------------------------------------------------- */
-int
-PutChar (SDL_Surface * Surface, int x, int y, unsigned char c)
-{
-  return PutCharFont (Surface, CurrentFont, x, y, c);
-}; // int PutChar (SDL_Surface * Surface, int x, int y, int c)
-
-/* ----------------------------------------------------------------------
+/**
  * Puts a single char on the surface with the specified font 
- * ---------------------------------------------------------------------- */
+ */
 int
 PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned char c)
 {
@@ -332,11 +213,6 @@ PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned ch
   if ( c<' ' || c>Font->number_of_chars-1 ) c = '.';
   if ( ( c != ' ' ) && ( c != '\n' ) )
     {
-      //--------------------
-      // In case of open gl output, we use our prepared small surfaces, since blitting
-      // only a part of the source image isn't effectively implemented in our SDL-OpenGL-wrapper
-      // function.
-      //
       if ( use_open_gl )
 	{
 	//--------------------
@@ -357,57 +233,53 @@ PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned ch
         glEnable(GL_CLIP_PLANE1);
 	#endif
 	if ( ( dest . x < clipping_rect . x + clipping_rect . w ) &&
-              ( dest . x >= clipping_rect . x ) )
-	      {
-	      if ( ! Font -> char_iso_image [ c ] . texture_has_been_created )
+		( dest . x >= clipping_rect . x ) )
+	    {
+	    if ( ! Font -> char_iso_image [ c ] . texture_has_been_created )
 		{
-		  if ( Font -> char_iso_image [ c ] . surface == NULL )
+		// If the character is not ready to be printed on screen
+		if ( Font -> char_iso_image [ c ] . surface == NULL )
 		    {
-		      GiveStandardErrorMessage ( __FUNCTION__  , "Surface was NULL pointer!",
-						 PLEASE_INFORM, IS_FATAL );
+		    ErrorMessage ( __FUNCTION__  , "Surface for character %d was NULL pointer!",
+			    PLEASE_INFORM, IS_FATAL, c );
 		    }
-		  make_texture_out_of_surface ( & ( Font -> char_iso_image [ c ] ) ) ;
-		  #ifdef HAVE_LIBGL
-		  glNewList(Font->list_base + c, GL_COMPILE);
+		make_texture_out_of_surface ( & ( Font -> char_iso_image [ c ] ) ) ;
+#ifdef HAVE_LIBGL
+		glNewList(Font->list_base + c, GL_COMPILE);
 
-	          glEnable( GL_TEXTURE_2D );
-		  glBindTexture(GL_TEXTURE_2D, *(Font->char_iso_image[c] . texture));
-	          glTexEnvi ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
-        	  glEnable ( GL_BLEND );
-        	  glDisable ( GL_ALPHA_TEST );
- 
-	          glBlendFunc( GL_SRC_ALPHA , GL_ONE_MINUS_SRC_ALPHA );
+		glBindTexture(GL_TEXTURE_2D, (Font->char_iso_image[c] . texture));
+		glTexEnvi ( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
+		glEnable ( GL_BLEND );
 
-		  glBegin(GL_QUADS);
+		glBegin(GL_QUADS);
 
-		  glTexCoord2f( 0, 1.0 );
-		  glVertex2i( 0 , 0 );
-		  glTexCoord2f( 0.0, 0 );
-		  glVertex2i( 0 , Font->char_iso_image[c] . texture_height );
-		  glTexCoord2f( 1.0, 0 );
-		  glVertex2i( Font->char_iso_image[c] . texture_width , Font->char_iso_image[c] . texture_height );
-		  glTexCoord2f( 1.0, 1 );
-		  glVertex2i(  Font->char_iso_image[c] . texture_width , 0 );
-		
-		  glEnd( );
-	          glDisable( GL_TEXTURE_2D );
-		  glEndList();
+		glTexCoord2f( 0.0, 1.0 );
+		glVertex2i( 0 , 0 );
+		glTexCoord2f( 0.0, 0.0 );
+		glVertex2i( 0 , Font->char_iso_image[c] . texture_height );
+		glTexCoord2f( 1.0, 0.0);
+		glVertex2i( Font->char_iso_image[c] . texture_width , Font->char_iso_image[c] . texture_height );
+		glTexCoord2f( 1.0, 1.0 );
+		glVertex2i(  Font->char_iso_image[c] . texture_width , 0 );
 
-		  #endif
+		glEnd( );
+		glEndList();
+		glDisable ( GL_BLEND );
+
+#endif
 		}
-	      
-	      //blit_open_gl_texture_to_screen_position ( & (Font -> char_iso_image [ c ]) , dest . x , dest . y , TRUE ) ;
-	      #ifdef HAVE_LIBGL
-	      glPushMatrix();
-	      glMatrixMode(GL_MODELVIEW);
-	      glTranslated(dest.x, dest.y, 0);
-	      glCallList(Font->list_base + c );
-	      glPopMatrix();
-              glDisable(GL_CLIP_PLANE0);
-	      glDisable(GL_CLIP_PLANE1);
-	      #endif
 
-              }
+#ifdef HAVE_LIBGL
+	    glPushMatrix();
+	    glMatrixMode(GL_MODELVIEW);
+	    glTranslated(dest.x, dest.y, 0);
+	    glCallList(Font->list_base + c );
+	    glPopMatrix();
+	    glDisable(GL_CLIP_PLANE0);
+	    glDisable(GL_CLIP_PLANE1);
+#endif
+
+	    }
 	}
       else
 	{
@@ -419,20 +291,20 @@ PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned ch
 
 }; // int PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, int c)
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 PutString (SDL_Surface * Surface, int x, int y, char *text)
 {
   PutStringFont (Surface, CurrentFont, x, y, text);
 }; // void PutString (SDL_Surface * Surface, int x, int y, char *text)
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 PutStringFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, char *text)
 {
@@ -467,20 +339,20 @@ PutStringFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, char *tex
     }
 }; // void PutStringFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, char *text)
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 int
 TextWidth (char *text)
 {
   return ( TextWidthFont (CurrentFont, text) ) ;
 }; // int TextWidth (char *text)
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 int
 TextWidthFont (BFont_Info * Font, char *text)
 {
@@ -512,6 +384,52 @@ TextWidthFont (BFont_Info * Font, char *text)
   return x;
 }; // int TextWidthFont (BFont_Info * Font, char *text)
 
+/**
+ *
+ *
+ */
+int
+LimitTextWidth (char *text, int limit)
+{
+  return ( LimitTextWidthFont (CurrentFont, text, limit) ) ;
+}; // int LimitTextWidth (char *text, int limit)
+
+/**
+ *
+ *
+ */
+int
+LimitTextWidthFont (BFont_Info * Font, char *text, int limit)
+{
+  int i = 0, x = 0;
+  //--------------------
+  // Based on Bastians hack: 
+  // 
+  // 'I added little hack to kern MenuFont..
+  // This basicly just prints them more tight on the screen.
+  // basse, 15.2.03'
+  //
+  // I extend this to give new text width results...
+  //
+  int kerning = 0;
+  if ( Font == Menu_BFont ) kerning = -4;
+  if ( Font==FPS_Display_BFont || Font==Blue_BFont || Font==Red_BFont) kerning = -2;
+
+  while (text[i] != '\0')
+    {
+      switch ( text [ i ] )
+	{
+	case 1: Font = Red_BFont; kerning = -2; break;
+	case 2: Font = Blue_BFont; kerning = -2; break;
+	case 3: Font = FPS_Display_BFont; kerning = -2; break;
+	}
+      x += CharWidth (Font, text[i]) + kerning ;
+      i++;
+      if (x >= limit ) return i;
+    }
+  return -1;
+}; // int LimitTextWidthFont (BFont_Info * Font, char *text, int limit)
+
 /* counts the spaces of the strings */
 int
 count (char *text)
@@ -932,10 +850,10 @@ PutPixel (SDL_Surface * surface, int x, int y, Uint32 pixel)
 
 }; // void PutPixel ( ... )
 
-/* ----------------------------------------------------------------------
+/**
  * NOTE:  I THINK THE SURFACE MUST BE LOCKED FOR THIS!
  *
- * ---------------------------------------------------------------------- */
+ */
 inline Uint32
 FdGetPixel32 (SDL_Surface * Surface, Sint32 X, Sint32 Y)
 {
