@@ -2,6 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -33,18 +34,14 @@
 #ifndef _defs_h
 #define _defs_h
 
-#ifndef _colodefs_h
-#include "colodefs.h"
-#endif
-
 //--------------------
 // Some technical definitions...
 //
 #ifndef TRUE
-#define TRUE (1==1)
+#define TRUE (1)
 #endif
 #ifndef FALSE
-#define FALSE (1==0)
+#define FALSE (0)
 #endif
 
 #define ERR 	-1
@@ -101,11 +98,6 @@ enum
     PART_GROUP_WEAPONARM = 5 ,
     ALL_PART_GROUPS = 6
   };
-enum
-  {
-    TUX_IMAGE_UPDATE_EVERYTHING_AT_ONCE = -133 ,
-    TUX_IMAGE_UPDATE_CONTINUOUSLY = -134 
-  };
 #define NOT_LOADED_MARKER "nothing_loaded"
 
 #define ENEMY_ROTATION_MODELS_AVAILABLE 35
@@ -124,8 +116,9 @@ enum
 #define NUMBER_OF_SHADOW_IMAGES 20
 
 
-#define MAX_OBSTACLES_ON_MAP 3000
+#define MAX_OBSTACLES_ON_MAP 4000
 #define MAX_OBSTACLE_NAMES_PER_LEVEL 100
+#define MAX_OBSTACLE_STATES_PER_LEVEL 300
 #define MAX_OBSTACLE_DESCRIPTIONS_PER_LEVEL 100
 
 #define FLOOR_TILES_VISIBLE_AROUND_TUX ((GameConfig . screen_width >= 1024 ? 12 : GameConfig . screen_width >= 800 ? 9 : 7))
@@ -143,8 +136,10 @@ enum
 #define NUMBER_OF_ITEM_PICTURES 70
 #define NUMBER_OF_MOUSE_CURSOR_PICTURES 2
 
-#define ITEM_DROP_PERCENTAGE 25
-#define GOLD_DROP_PERCENTAGE 15
+#define ITEM_DROP_PERCENTAGE 15
+#define GOLD_DROP_PERCENTAGE 20
+#define ARMOUR_DURABILITYLOSS_PERCENTAGE_WHEN_HIT 20
+#define WEAPON_DURABILITYLOSS_PERCENTAGE_WHEN_USED 15
 
 #define MAX_DIALOGUE_OPTIONS_IN_ROSTER 100
 #define MAX_REPLIES_PER_OPTION 100
@@ -154,15 +149,6 @@ enum
 
 #define MAX_BIG_SCREEN_MESSAGES 10
 
-
-#ifndef _itemdefs_h
-#include "itemdefs.h"
-#endif
-
-#define WAR_BOT 1
-#define SNIPER_BOT 2
-#define MIND_BOT 3
-
 #define NUMBER_OF_SKILL_LEVELS 10
 #define NUMBER_OF_SKILL_PAGES 8
 #define NUMBER_OF_SKILLS_PER_SKILL_PAGE 5
@@ -171,7 +157,7 @@ enum
 
 enum skill_forms
 {
-    PROGRAM_FORM_IMMEDIATE = 60000, //instant hit whatever behind cursor
+    PROGRAM_FORM_IMMEDIATE = 10, //instant hit whatever behind cursor
     PROGRAM_FORM_RADIAL, //nova
     PROGRAM_FORM_BULLET, //fire a bullet
     PROGRAM_FORM_SELF, //instant hit player
@@ -182,13 +168,10 @@ enum _global_ingame_modes
     GLOBAL_INGAME_MODE_IDENTIFY = 7123 ,
     GLOBAL_INGAME_MODE_NORMAL = 7124 ,
     GLOBAL_INGAME_MODE_EXAMINE = 7125 ,
-    GLOBAL_INGAME_MODE_LOOT = 7126 ,
     GLOBAL_INGAME_MODE_SCROLL_UP = 7127, 
     GLOBAL_INGAME_MODE_SCROLL_DOWN = 7128 ,
     GLOBAL_INGAME_MODE_REPAIR = 7129 ,
     GLOBAL_INGAME_MODE_UNLOCK = 7130 ,
-    GLOBAL_INGAME_MODE_TALK = 7131 ,
-    GLOBAL_INGAME_MODE_FIRST_AID = 7132 ,
     GLOBAL_INGAME_MODE_ATTACK = 7133 ,
     GLOBAL_INGAME_MODE_PICKPOCKET = 7134 ,
 };
@@ -201,11 +184,6 @@ enum _game_config_screen_visible_parameters
     GAME_CONFIG_SCREEN_VISIBLE_SKILL_EXPLANATION = 8004 ,
 };
 
-//--------------------
-// This is old and should go out, but that must be done from at home, 
-// cause here I can't check if the code is still running without it
-//
-#define FILENAME_LEN	5000
 
 #define END_OF_OFFSET_FILE_STRING "** End of iso_image offset file **"
 #define START_OF_OFFSET_FILE_STRING "** Start of iso_image offset file **"
@@ -231,12 +209,35 @@ enum _game_config_screen_visible_parameters
 enum
 { ONLY_SHOW_MAP = 1 , DO_SCREEN_UPDATE = 2 , ONLY_SHOW_MAP_AND_TEXT = 4 , SHOW_GRID = 8 , SHOW_TUX_AND_ENEMIES = 16 , SHOW_TEXT = 32 , SHOW_ITEMS = 64 , OMIT_OBSTACLES = 128 , OMIT_TUX = 256 , OMIT_ENEMIES = 512 , SHOW_OBSTACLE_NAMES = 1024 , ZOOM_OUT = 2048 , OMIT_BLASTS = 4096 , USE_OWN_MOUSE_CURSOR = 8192 , SKIP_LIGHT_RADIUS = (2*8192) };
 
+// Flags for obstacles
+enum 
+{
+	/* Obstacle groups */
+	IS_WALL = 1,
+
+	/* Walls */
+	IS_HORIZONTAL = 2,
+	IS_VERTICAL = 4,
+
+	/* Obstacle specs */
+	BLOCKS_VISION_TOO = 8,			/* Light will not pass through this obstacle */
+	IS_SMASHABLE = 16,
+	DROPS_RANDOM_TREASURE = 32,
+	NEEDS_PRE_PUT = 64,
+	GROUND_LEVEL = 128,			/* This obstacle has a small height */
+	IS_WALKABLE = 256			/* A bot can traverse this obstacle */
+};
+
+
+
+
 //--------------------
 // This controls the zoom factor for the automap.  Since this uses
 // a different update policy than the level editor, even strong zoom
 // will not be a problem here...
 //
-#define LEVEL_EDITOR_ZOOM_OUT_FACT 3.0
+#define LEVEL_EDITOR_ZOOM_OUT_FACT 2
+#define ONE_OVER_LEVEL_EDITOR_ZOOM_OUT_FACT 0.5
 
 #define AUTOMAP_ZOOM_OUT_FACT 8.0
 
@@ -291,7 +292,6 @@ enum
 // #define LEVEL_EDITOR_BANNER_CODE6 19 
 // #define LEVEL_EDITOR_BANNER_CODE7 20 
 #define FREEDROID_LOADING_PICTURE_CODE 21
-#define MOUSE_BUTTON_CHA_BACKGROUND_PICTURE_CODE 22
 #define MOUSE_BUTTON_INV_BACKGROUND_PICTURE_CODE 23
 #define MOUSE_BUTTON_SKI_BACKGROUND_PICTURE_CODE 24
 #define MOUSE_BUTTON_PLUS_BACKGROUND_PICTURE_CODE 25
@@ -325,8 +325,8 @@ enum
 #define BLOCKMEM  		INITIAL_BLOCK_WIDTH*INITIAL_BLOCK_HEIGHT
 #define MAX_AMOUNT_OF_TEXTURES_WE_WILL_USE 10000
 
-#define SCROLLSTARTX		User_Rect.x
-#define SCROLLSTARTY		GameConfig . screen_height - 100
+#define TABCHAR 'W'
+#define TABWIDTH 8
 
 #define INITIAL_DIGIT_LENGTH (8*2)
 #define INITIAL_DIGIT_HEIGHT (9*2)
@@ -342,7 +342,7 @@ enum
 #define LEFT_INFO_Y	(10*2)
 #define RIGHT_INFO_X	(242*2)
 #define RIGHT_INFO_Y	(8*2)
-#define LEFT_TEXT_LEN 10
+#define LEFT_TEXT_LEN 20
 #define RIGHT_TEXT_LEN 6
 #define TEXT_STRETCH (6.0/5.0)
 
@@ -381,12 +381,6 @@ enum _sounds
 // occur periodically.  These constants specify which intervals are to be used
 // for these periodic happenings...
 #define CRY_SOUND_INTERVAL 2
-#define TRANSFER_SOUND_INTERVAL 1.1
-
-// #define MAX_PLAYERS             5
-#define MAX_PLAYERS             1
-#define MAX_PLAYERS_AT_MOST     5
-#define PLAYER_NR_0             0
 
 enum _non_player_characters
   {
@@ -400,7 +394,7 @@ enum _non_player_characters
     PERSON_614,
     PERSON_MER,
     PERSON_FRANCIS,
-    PERSON_ERNIE,
+    PERSON_FIRMWARE_SERVER,
     PERSON_BENJAMIN,
     PERSON_BENDER,
     PERSON_SPENCER,
@@ -432,6 +426,8 @@ enum _non_player_characters
     PERSON_BORIS,
     PERSON_LINA,
     PERSON_SERGE,
+    PERSON_TEST_DROID,
+    PERSON_MSCD,
     LAST_PERSON
   };
 #define MAX_PERSONS             (LAST_PERSON)
@@ -452,39 +448,6 @@ enum
     SKI_SCREEN_TOGGLE_BUTTON_YELLOW, 
     UP_BUTTON,
     DOWN_BUTTON,
-    LEFT_BUTTON,
-    RIGHT_BUTTON,
-    MAP_EXIT_BUTTON,
-    MAP_UNLOCK_BUTTON_GREEN,
-    MAP_UNLOCK_BUTTON_YELLOW,
-    MAP_UNLOCK_BUTTON_RED,
-    MAP_GUNOFF_BUTTON_GREEN,
-    MAP_GUNOFF_BUTTON_YELLOW,
-    MAP_GUNOFF_BUTTON_RED,
-    MAP_SECURITYMIDDLE_BUTTON,
-    MAP_SECURITYLEFT_BUTTON,
-    MAP_SECURITYRIGHT_BUTTON,
-    MAP_PASSWORDMIDDLE_BUTTON,
-    MAP_PASSWORDLEFT_BUTTON,
-    MAP_PASSWORDRIGHT_BUTTON,
-    MAP_REQUEST_ENERGY_RATION_GREEN_BUTTON,
-    MAP_REQUEST_ENERGY_RATION_RED_BUTTON,
-    MAP_READ_EMAIL_GREEN_BUTTON,
-    MAP_READ_EMAIL_RED_BUTTON,
-    MAP_GUN_TYPE_1_BUTTON_RED,
-    MAP_GUN_TYPE_2_BUTTON_RED,
-    MAP_GUN_TYPE_3_BUTTON_RED,
-    MAP_GUN_TYPE_4_BUTTON_RED,
-    MAP_GUNON_BUTTON_GREEN,
-    MAP_GUNON_BUTTON_YELLOW,
-    MAP_GUNON_BUTTON_RED,
-    MAP_GUN_TYPE_1_BUTTON_YELLOW,
-    MAP_GUN_TYPE_2_BUTTON_YELLOW,
-    MAP_GUN_TYPE_3_BUTTON_YELLOW,
-    MAP_GUN_TYPE_4_BUTTON_YELLOW,
-    CONSOLE_IDENTIFY_BUTTON_GREEN,
-    CONSOLE_IDENTIFY_BUTTON_RED,
-    CONSOLE_IDENTIFY_BUTTON_YELLOW,
     ITEM_BROWSER_LEFT_BUTTON,
     ITEM_BROWSER_RIGHT_BUTTON,
     ITEM_BROWSER_EXIT_BUTTON,
@@ -502,55 +465,93 @@ enum
 
     BUY_BUTTON,
     SELL_BUTTON,
-    TAKE_BUTTON,
-    PUT_BUTTON,
     REPAIR_BUTTON,
     IDENTIFY_BUTTON,
 
     OPEN_CLOSE_SKILL_EXPLANATION_BUTTON,
 
     GO_LEVEL_NORTH_BUTTON,
+    GO_LEVEL_NORTH_BUTTON_PUSHED,
     GO_LEVEL_SOUTH_BUTTON,
+    GO_LEVEL_SOUTH_BUTTON_PUSHED,
     GO_LEVEL_EAST_BUTTON,
+    GO_LEVEL_EAST_BUTTON_PUSHED,
     GO_LEVEL_WEST_BUTTON,
+    GO_LEVEL_WEST_BUTTON_PUSHED,
     EXPORT_THIS_LEVEL_BUTTON,
+    EXPORT_THIS_LEVEL_BUTTON_PUSHED,
     LEVEL_EDITOR_SAVE_SHIP_BUTTON,
+    LEVEL_EDITOR_SAVE_SHIP_BUTTON_PUSHED,
+    LEVEL_EDITOR_SAVE_SHIP_BUTTON_OFF,
+    LEVEL_EDITOR_SAVE_SHIP_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON,
+    LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON,
+    LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_CONNECTION_RED_BUTTON,
+    LEVEL_EDITOR_TOGGLE_CONNECTION_RED_BUTTON_PUSHED,
 
     LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON,
+    LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON_PUSHED,
     LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON,
+    LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON_PUSHED,
 
     LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON,
+    LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON_PUSHED,
     LEVEL_EDITOR_ZOOM_IN_BUTTON,
+    LEVEL_EDITOR_ZOOM_IN_BUTTON_PUSHED,
     LEVEL_EDITOR_ZOOM_OUT_BUTTON,
+    LEVEL_EDITOR_ZOOM_OUT_BUTTON_PUSHED,
     LEVEL_EDITOR_RECURSIVE_FILL_BUTTON,
+    LEVEL_EDITOR_RECURSIVE_FILL_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON,
+    LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_OBSTACLE_DESCRIPTION_BUTTON,
+    LEVEL_EDITOR_NEW_OBSTACLE_DESCRIPTION_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON,
+    LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON_PUSHED,
     LEVEL_EDITOR_NEW_ITEM_BUTTON,
+    LEVEL_EDITOR_NEW_ITEM_BUTTON_PUSHED,
     LEVEL_EDITOR_ESC_BUTTON,
+    LEVEL_EDITOR_ESC_BUTTON_PUSHED,
     LEVEL_EDITOR_LEVEL_RESIZE_BUTTON,
+    LEVEL_EDITOR_LEVEL_RESIZE_BUTTON_PUSHED,
     LEVEL_EDITOR_KEYMAP_BUTTON,
+    LEVEL_EDITOR_KEYMAP_BUTTON_PUSHED,
     LEVEL_EDITOR_QUIT_BUTTON,
+    LEVEL_EDITOR_QUIT_BUTTON_PUSHED,
 
     LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON,
+    LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON_PUSHED,
     LEVEL_EDITOR_UNDERGROUND_LIGHT_OFF_BUTTON,
+    LEVEL_EDITOR_UNDERGROUND_LIGHT_OFF_BUTTON_PUSHED,
 
     LEVEL_EDITOR_TOGGLE_TUX_BUTTON,
+    LEVEL_EDITOR_TOGGLE_TUX_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_TUX_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_TUX_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON,
+    LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON,
+    LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_OBSTACLES_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON,
+    LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_OFF,
-    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON ,
+    LEVEL_EDITOR_TOGGLE_TOOLTIPS_BUTTON_OFF_PUSHED,
+    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON,
+    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_COLLISION_RECTS_BUTTON_OFF_PUSHED,
     LEVEL_EDITOR_TOGGLE_GRID_BUTTON,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_PUSHED,
     LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_FULL_PUSHED,
     LEVEL_EDITOR_TOGGLE_GRID_BUTTON_OFF,
+    LEVEL_EDITOR_TOGGLE_GRID_BUTTON_OFF_PUSHED,
 
     LEVEL_EDITOR_NEXT_ITEM_GROUP_BUTTON,
     LEVEL_EDITOR_PREV_ITEM_GROUP_BUTTON,
@@ -559,6 +560,10 @@ enum
     LEVEL_EDITOR_NEXT_SUFFIX_BUTTON,
     LEVEL_EDITOR_PREV_SUFFIX_BUTTON,
     LEVEL_EDITOR_CANCEL_ITEM_DROP_BUTTON,
+    LEVEL_EDITOR_UNDO_BUTTON,
+    LEVEL_EDITOR_UNDO_BUTTON_PUSHED,
+    LEVEL_EDITOR_REDO_BUTTON,
+    LEVEL_EDITOR_REDO_BUTTON_PUSHED,
 
     SAVE_GAME_BANNER,
     LOAD_GAME_BANNER,
@@ -566,8 +571,6 @@ enum
     WEAPON_RECT_BUTTON,
     DRIVE_RECT_BUTTON,
     SHIELD_RECT_BUTTON,
-    AUX1_RECT_BUTTON,
-    AUX2_RECT_BUTTON,
     HELMET_RECT_BUTTON,
     ARMOUR_RECT_BUTTON,
 
@@ -626,17 +629,17 @@ enum
     MAX_MOUSE_PRESS_BUTTONS
   };
 
-#define CHAT_SUBDIALOG_WINDOW_X ((260)*GameConfig . screen_width/640)
-#define CHAT_SUBDIALOG_WINDOW_Y ((42)*GameConfig . screen_height/480)
-#define CHAT_SUBDIALOG_WINDOW_W ((337)*GameConfig . screen_width/640)
-#define CHAT_SUBDIALOG_WINDOW_H ((216)*GameConfig . screen_height/480)
+#define CHAT_SUBDIALOG_WINDOW_X ((260)*GameConfig . screen_width/640.0)
+#define CHAT_SUBDIALOG_WINDOW_Y ((42)*GameConfig . screen_height/480.0)
+#define CHAT_SUBDIALOG_WINDOW_W ((337)*GameConfig . screen_width/640.0)
+#define CHAT_SUBDIALOG_WINDOW_H ((216)*GameConfig . screen_height/480.0)
 
 #define STR_X 95
-#define STR_Y 141
+#define STR_Y 140
 #define DEX_Y 165
 #define MAG_Y 190
-#define VIT_Y 218
-#define POINTS_Y 252
+#define VIT_Y 215
+#define POINTS_Y 245
 
 #define BUTTON_WIDTH 35
 #define BUTTON_HEIGHT 19
@@ -661,7 +664,7 @@ enum
 #define TUX_RUN_CYCLE_PHASES   10
 #define TUX_TOTAL_PHASES (TUX_SWING_PHASES+TUX_BREATHE_PHASES+TUX_GOT_HIT_PHASES+TUX_WALK_CYCLE_PHASES+TUX_RUN_CYCLE_PHASES)
 
-#define TUX_RUNNING_SPEED (7.1)
+#define TUX_RUNNING_SPEED (5.5)
 #define TUX_WALKING_SPEED (2.5)
 
 #define TUX_MODELS              13
@@ -672,12 +675,13 @@ enum
 #define WAIT_AFTER_KILLED	3.0  // time to wait and still display pictures after the destruction of 
                                      // the players droid.  This is now measured in seconds and can be a float
 #define WAIT_AFTER_GAME_WON	5.0  // how long to show tux after the game is won
-#define WAIT_COLLISION		2.5 // after a little collision with Tux or another enemy, hold position for a while
+#define WAIT_COLLISION		1 // after a little collision with Tux or another enemy, hold position for a while
                                     // this variable describes the amount of time in SECONDS 
 #define FLASH_DURATION_IN_SECONDS 0.1
 
 #define ALLBLASTTYPES		3	/* number of different exposions */
 
+#define MAX_MELEE_SHOTS		100
 #define MAXBULLETS		100	/* maximum possible Bullets in the air */
 #define MAXBLASTS		100	/* max. possible Blasts visible */
 #define AGGRESSIONMAX  		100
@@ -707,37 +711,136 @@ enum
 
 #define MAX_SAVED_CHARACTERS_ON_DISK 7
 
-#define MAX_ENEMYS_ON_SHIP	1000
-#define MAX_INFLU_POSITION_HISTORY 10000
-#define MAX_MISSIONS_IN_GAME 12         // how many missions can there be at most in Freedroid
+#define MAX_INFLU_POSITION_HISTORY 500
+#define MAX_MISSIONS_IN_GAME 15         // how many missions can there be at most in Freedroid
 #define MAX_MISSION_TRIGGERED_ACTIONS 10
 #define MAX_MISSION_DESCRIPTION_TEXTS 10
 
 #define MAX_LEVELS		39   	// how many map levels are allowed in one ship
 
-#define MAX_EVENT_TRIGGERS      100   // how many event triggers at most to allow
-#define MAX_TRIGGERED_ACTIONS_IN_GAME   100   // how many triggerable actions to allow at most
+#define MAX_EVENT_TRIGGERS      500   // how many event triggers at most to allow
+#define MAX_TRIGGERED_ACTIONS_IN_GAME   500   // how many triggerable actions to allow at most
 
-#define MAXWAYPOINTS 		200
-#define MAX_DOORS_ON_LEVEL 	120
+#define MAXWAYPOINTS 		220
+#define MAX_DOORS_ON_LEVEL 	150
 #define MAX_AUTOGUNS_ON_LEVEL 	60
 #define MAX_REFRESHES_ON_LEVEL	39
 #define MAX_TELEPORTERS_ON_LEVEL 20
 
 #define MAX_PHASES_IN_A_BULLET 12
 #define PHASES_OF_EACH_BLAST 20
-#define MAX_STEPS_IN_GIVEN_COURSE 100
-
-#undef USE_MISS_HIT_ARRAYS
-#define UNCHECKED 0
-#define HIT 1
-#define MISS 2
 
 #define ITEM_INVENTORY_IMAGE_FILE_NAME "File or directory name for inventory image=\""
 #define ITEM_DROP_SOUND_FILE_NAME "Item uses drop sound with filename=\""
 
-#define UNIVERSAL_COORD_W(W) (int)((float)W * ((float)(GameConfig . screen_width) / 640.0))
-#define UNIVERSAL_COORD_H(H) (int)((float)H * ((float)(GameConfig . screen_height) / 480.0))
+#define UNIVERSAL_COORD_W(W) (int)((float)(W) * ((float)(GameConfig . screen_width) / 640.0))
+#define UNIVERSAL_COORD_H(H) (int)((float)(H) * ((float)(GameConfig . screen_height) / 480.0))
+
+//--------------------
+// And now we put here all the rectangle definitions and that...
+//
+
+// Game message window
+#define SUBTITLEW_RECT_X 120
+#define SUBTITLEW_RECT_Y 389
+#define SUBTITLEW_RECT_W 415
+#define SUBTITLEW_RECT_H 44
+
+// the inventory rect is the rectangle inside the inventory screen,
+// where all the items "in the backpack" including the quick inventory
+// are displayed.  it's relative to the top left corner, as always.
+//
+
+#define INVENTORY_RECT_Y 260
+#define INVENTORY_RECT_X 8
+
+#define CURRENT_SKILL_RECT_X 537
+#define CURRENT_SKILL_RECT_Y (GameConfig . screen_height - 80 * GameConfig . screen_height / 480)
+#define CURRENT_SKILL_RECT_W 64
+#define CURRENT_SKILL_RECT_H 64
+
+// 388 is the maximum y size to fit items up to 6 inventory squares high
+// 398 is the maximum y size to fit items up to 5 inventory squares high
+#define CURRENT_WEAPON_RECT_X 35
+#define CURRENT_WEAPON_RECT_Y 398
+#define CURRENT_WEAPON_RECT_W 64
+#define CURRENT_WEAPON_RECT_H 64
+
+#define WHOLE_HEALTH_RECT_Y (480-85)
+#define WHOLE_HEALTH_RECT_W 6
+#define WHOLE_HEALTH_RECT_H 50
+#define WHOLE_HEALTH_RECT_X 614
+#define HEALTH_RECT_COLOR (63*64*64 + 64*8)
+
+#define GOLDEN_SCHNITT ((sqrt(5)-1)/2)
+
+#define WHOLE_RUNNING_POWER_RECT_X 6
+#define WHOLE_RUNNING_POWER_RECT_Y (480-85)
+#define WHOLE_RUNNING_POWER_RECT_W 6
+#define WHOLE_RUNNING_POWER_RECT_H 50
+
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_X 19
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_Y WHOLE_RUNNING_POWER_RECT_Y
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_W WHOLE_RUNNING_POWER_RECT_W
+#define WHOLE_EXPERIENCE_COUNTDOWN_RECT_H WHOLE_RUNNING_POWER_RECT_H
+
+#define WHOLE_FORCE_RECT_Y ( WHOLE_HEALTH_RECT_Y )
+#define WHOLE_FORCE_RECT_W ( WHOLE_HEALTH_RECT_W )
+#define WHOLE_FORCE_RECT_H ( WHOLE_HEALTH_RECT_H )
+#define WHOLE_FORCE_RECT_X ( 640-12)
+#define FORCE_RECT_COLOR ( 0 * 64 * 64 + 64 * 0 + 63 )
+
+//--------------------
+// The weapon rectangle is of course for the weapon the Tux is
+// using.  When using a 2-handed weapon, the 'shield' rectangle
+// will ALSO be used to show this weapon, since the shield-hand
+// is also needed to wield such a thing.
+//
+// 31 is the minimum x size to fit items up to 4 inventory squares wide
+#define WEAPON_RECT_WIDTH 67
+#define WEAPON_RECT_HEIGHT 112
+#define WEAPON_RECT_X 31
+#define WEAPON_RECT_Y 117
+
+//--------------------
+// The armour rectangle is where you can place some armour
+// for the body of the Tux.
+//
+#define ARMOUR_RECT_WIDTH 66
+#define ARMOUR_RECT_HEIGHT 92
+#define ARMOUR_RECT_X 130
+#define ARMOUR_RECT_Y 70
+
+//--------------------
+// The shield rectangle is obviously for the shields, but not only
+// that:  when using a 2-handed weapon, its also good for the weapon
+// again, to indicate, that no shield can be used in conjunction with
+// such a 2-handed weapon.
+//
+#define SHIELD_RECT_WIDTH 70
+#define SHIELD_RECT_HEIGHT 112
+#define SHIELD_RECT_X 230
+#define SHIELD_RECT_Y 116
+
+//--------------------
+// The 'drive' rectangle is actually for the footwear.
+//
+#define DRIVE_RECT_WIDTH 64
+#define DRIVE_RECT_HEIGHT 64
+#define DRIVE_RECT_X 130
+#define DRIVE_RECT_Y 175
+
+//--------------------
+// Now this is the 'helmet' rectangle.
+//
+#define HELMET_RECT_WIDTH 64
+#define HELMET_RECT_HEIGHT 64
+#define HELMET_RECT_X 25
+#define HELMET_RECT_Y 22
+
+// Flags for thrown items. see PutItem function in view.c
+#define PUT_ONLY_THROWN_ITEMS 3
+#define PUT_NO_THROWN_ITEMS 4
 
 
 //--------------------
@@ -789,18 +892,7 @@ enum _status
 {
     INFOUT=-30,
     MOBILE,
-    TRANSFERMODE,
-    WEAPON,
-    CAPTURED,
-    COMPLETE,
-    REJECTED,
-    CONSOLE,
-    DEBRIEFING,
-    PAUSE,
-    CHEESE,
-    ELEVATOR,
     BRIEFING,
-    MENU
 };
 
 enum _busytype
@@ -812,6 +904,14 @@ enum _busytype
     THROWING_GRENADE,
     RUNNING_PROGRAM,
 };
+
+enum _attackhit
+{
+    ATTACK_HIT_BOTS=1,
+    ATTACK_HIT_HUMANS=2,
+    ATTACK_HIT_ALL=3,
+};
+
 //--------------------
 // The possible edito modes for the level editor
 //
@@ -824,30 +924,41 @@ enum _level_editor_edit_modes
     LEVEL_EDITOR_SELECTION_CONTAINERS = 4,
     LEVEL_EDITOR_SELECTION_PLANTS = 5,
     LEVEL_EDITOR_SELECTION_ALL = 6,
-    NUMBER_OF_LEVEL_EDITOR_GROUPS = 7
+    LEVEL_EDITOR_SELECTION_QUICK = 7, 
+    NUMBER_OF_LEVEL_EDITOR_GROUPS = 8
 };
 
+// possible directions for a line of walls
+// We use 1 et -1, because north is the opposite of south.
+enum _level_editor_directions
+{
+    UNDEFINED = 0,
+    NORTH = 1,
+    EAST = 2,
+    SOUTH = -1,
+    WEST = -2
+};
+
+
 enum _enemy_combat_state_machine_states
 {
-    MOVE_ALONG_RANDOM_WAYPOINTS = -301 , 
-    STOP_AND_EYE_TUX = -302 ,
-    MAKE_ATTACK_RUN = -303 , 
-    HUNTING_TUX = -304 , 
-    SEEK_BETTER_POSITION = -305 ,
-    FIGHT_ON_TUX_SIDE = -306 ,
-    RELENTLESS_FIRE_TO_GIVEN_POSITION = -307 ,
-    TURN_THOWARDS_NEXT_WAYPOINT = -308 ,
-    RUSH_TUX_ON_SIGHT_AND_OPEN_TALK = -309 ,
-    WAIT_AND_TURN_AROUND_AIMLESSLY = -310 ,
-    WAYPOINTLESS_WANDERING = -311 ,
-    TURN_THOWARDS_WAYPOINTLESS_SPOT = -312 
-    // BACK_OFF_AFTER_GETTING_HIT, 
+    MOVE_ALONG_RANDOM_WAYPOINTS, 
+    STOP_AND_EYE_TARGET,
+    ATTACK, 
+    PARALYZED,
+    COMPLETELY_FIXED,
+    FOLLOW_TUX,
+    RETURNING_HOME,
+    SELECT_NEW_WAYPOINT,
+    TURN_TOWARDS_NEXT_WAYPOINT,
+    RUSH_TUX_AND_OPEN_TALK,
+    WAYPOINTLESS_WANDERING,
 };
 enum _enemy_combat_target_codes
 {
-    ATTACK_TARGET_IS_ENEMY = -201,
-    ATTACK_TARGET_IS_PLAYER = -202,
-    ATTACK_TARGET_IS_NOTHING = -203
+    ATTACK_TARGET_IS_ENEMY = -101,
+    ATTACK_TARGET_IS_PLAYER = -102,
+    ATTACK_TARGET_IS_NOTHING = -103
 };
 enum
 {
@@ -855,23 +966,12 @@ enum
 }; 
 enum
 {
-    TRANSPARENCY_FOR_WALLS = -501 ,
-    TRANSPARENCY_FOR_SEE_THROUGH_OBJECTS = -502
+    TRANSPARENCY_NONE = 0,
+    TRANSPARENCY_FOR_WALLS = 10 ,
+    TRANSPARENCY_FOR_SEE_THROUGH_OBJECTS = 11,
+    TRANSPARENCY_CUROBJECT = 12
 }; 
 
-
-//--------------------
-// The possible networking statuses of clients.
-//
-enum _networking_status
-{
-    NETWORK_ERROR = 0 ,
-    UNCONNECTED , 
-    CONNECTION_FRESHLY_OPENED ,
-    NAME_HAS_BEEN_TRANSMITTED , 
-    GAME_ON
-};
-
 //--------------------
 // It is possible, that the Tux must fist move somewhere AND
 // then open a chest or move somewhere AND then pick something up
@@ -884,7 +984,6 @@ enum _combo_action_types
     COMBO_ACTION_OPEN_CHEST = 15 ,
     COMBO_ACTION_PICK_UP_ITEM = 16 ,
     COMBO_ACTION_SMASH_BARREL = 17 ,
-    COMBO_ACTION_LOOT_OBSTACLE = 18 ,
 };
 
 //--------------------
@@ -895,17 +994,7 @@ enum
   {
     DO_NOTHING = -1 , 
     BUY_1_ITEM = 1 , 
-    BUY_10_ITEMS = 2 , 
-    BUY_100_ITEMS = 3 , 
     SELL_1_ITEM = 4 , 
-    SELL_10_ITEMS = 5 , 
-    SELL_100_ITEMS = 6 , 
-    PUT_1_ITEM = 6 , 
-    PUT_10_ITEMS = 7 , 
-    PUT_100_ITEMS = 8 , 
-    TAKE_1_ITEM = 9 , 
-    TAKE_10_ITEMS = 10 , 
-    TAKE_100_ITEMS = 11 ,
     REPAIR_ITEM = 12 , 
     IDENTIFY_ITEM = 13 
   };
@@ -1401,6 +1490,12 @@ enum
   ISO_GREY_WALL_CORNER_2,
   ISO_GREY_WALL_CORNER_3,
   ISO_GREY_WALL_CORNER_4,
+  
+  ISO_BROKEN_GLASS_WALL_1,
+  ISO_OUTER_DOOR_V_OFFLINE ,
+  ISO_OUTER_DOOR_H_OFFLINE ,
+  ISO_H_DDOOR_LOCKED ,
+
 
   NUMBER_OF_OBSTACLE_TYPES
 };
@@ -1502,5 +1597,6 @@ enum
   ALL_ISOMETRIC_FLOOR_TILES 
 };
 
+#define ITEM_TAKE_DIST (1.2)
 
 #endif
