@@ -2,6 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -28,7 +29,7 @@
 #define _global_h
 
 #include "BFont.h"
-#include "colodefs.h"
+#include "lists.h"
 
 #undef EXTERN
 #ifdef _main_c
@@ -38,31 +39,15 @@
 
 EXTERN char* floor_tile_filenames [ ALL_ISOMETRIC_FLOOR_TILES ] ;
 EXTERN obstacle_spec obstacle_map[ NUMBER_OF_OBSTACLE_TYPES ];
-// EXTERN item_image_spec ItemImageList[ NUMBER_OF_ITEM_PICTURES ];
-EXTERN char EndTitleText1[];
-EXTERN char EndTitleText2[];
-EXTERN char TitleText1[];
-EXTERN char TitleText2[];
-EXTERN char TitleText3[];
-EXTERN char TitleText4[];
 EXTERN float FPSover1;
 EXTERN float FPSover10;
 EXTERN float FPSover100;
-EXTERN char *NetworkClientStatusNames[];
-EXTERN char *Classname[];
-EXTERN char *Classes[];
-EXTERN char *Height[];
-EXTERN char *Weight[];
-EXTERN char *Entry[];
-EXTERN char *Weaponnames[];
-EXTERN char *Sensornames[];
 EXTERN char *AllSkillTexts[];
-EXTERN char *SkillName[];
-EXTERN char *Brainnames[];
-EXTERN int ThisMessageTime;
 EXTERN char font_switchto_red [ 2 ] ;
 EXTERN char font_switchto_blue [ 2 ] ;
 EXTERN char font_switchto_neon [ 2 ] ;
+EXTERN char font_switchto_msgstat [ 2 ] ;
+EXTERN char font_switchto_msgvar [ 2 ] ;
 
 EXTERN int SpellHitPercentageTable [  ] ;
 EXTERN float MeleeDamageMultiplierTable [  ] ;
@@ -70,67 +55,48 @@ EXTERN float MeleeRechargeMultiplierTable [  ] ;
 EXTERN float RangedDamageMultiplierTable [  ] ;
 EXTERN float RangedRechargeMultiplierTable [  ] ;
 EXTERN spell_skill_spec * SpellSkillMap ;
-EXTERN tux_t Me[ MAX_PLAYERS ];		/* the influence data */
-EXTERN network_tux_t NetworkMe[ MAX_PLAYERS ];		/* the influence data */
+EXTERN tux_t Me; /* the influence data */
 EXTERN Druidspec Druidmap;     
 EXTERN Bulletspec Bulletmap;
 EXTERN blastspec Blastmap[ALLBLASTTYPES];
 EXTERN int skip_initial_menus;
 EXTERN int number_of_skills;
+EXTERN supported_languages_t supported_languages[];
+EXTERN int last_bot_number;
+EXTERN enum { INSIDE_MENU = 0, INSIDE_GAME, INSIDE_LVLEDITOR } game_status;
 #endif
 
 EXTERN SDL_Rect User_Rect;
-EXTERN SDL_Rect Full_Screen_Rect;
-EXTERN SDL_Rect Classic_User_Rect;
 EXTERN SDL_Rect Full_User_Rect;
-EXTERN SDL_Rect Cons_Rect;
 EXTERN SDL_Rect Cons_Text_Rect;
 
 extern char *our_homedir;
 extern char *our_config_dir;
 
-EXTERN const SDL_Rect Menu_Rect;
-
-
 EXTERN int Number_Of_Droid_Types;
 EXTERN int QuitProgram;
 EXTERN int GameOver;
-EXTERN int AlertLevel;
-EXTERN int RespectVisibilityOnMap;
 
-EXTERN int ServerMode;
-EXTERN int ClientMode;
+extern list_head_t alive_bots_head;
+extern list_head_t dead_bots_head;
+extern list_head_t level_bots_head[MAX_LEVELS];
 
-EXTERN enemy AllEnemys[ MAX_ENEMYS_ON_SHIP ];
 EXTERN spell_active AllActiveSpells[ MAX_ACTIVE_SPELLS ];
-EXTERN network_enemy NetworkAllEnemys[ MAX_ENEMYS_ON_SHIP ];
 EXTERN event_trigger AllEventTriggers[ MAX_EVENT_TRIGGERS ];
 EXTERN triggered_action AllTriggeredActions[ MAX_TRIGGERED_ACTIONS_IN_GAME ];
-EXTERN char ServerName[ 10000 ];
 
-EXTERN Level CurLevel;		/* the current level data */
 EXTERN ship curShip;		/* the current ship-data */
 
 EXTERN bullet AllBullets[MAXBULLETS + 10];
+EXTERN melee_shot AllMeleeShots[MAX_MELEE_SHOTS];
 EXTERN blast AllBlasts[MAXBLASTS + 10];
-EXTERN int KeyCode;
-
-EXTERN unsigned int MapBlockIndex;
-
-EXTERN unsigned char *InternWindow;
-
-EXTERN int taste;
 
 EXTERN int sound_on;		// Toggle TRUE/FALSE for turning sounds on/off 
 EXTERN int debug_level;       	// 0=no debug 1=some debug messages 2=...etc 
 				// (currently only 0 or !=0 is implemented) 
 EXTERN int show_all_droids;     // display enemys regardless of IsVisible() 
-EXTERN int stop_influencer;     // for bullet debugging: stop where u are 
-EXTERN int xray_vision_for_tux; // to better debug bot movements from behind walls...
 EXTERN int draw_collision_rectangles; // to better debug collision rectangles
 EXTERN int draw_grid;           // grid to see where objects will be positioned
-EXTERN int mouse_control;       // allow for mouse control 
-EXTERN int classic_user_rect;   // use the User-Rect dimensions of the original game? 
 #undef EXTERN
 #ifdef _misc_c
 #define EXTERN
@@ -138,13 +104,6 @@ EXTERN int classic_user_rect;   // use the User-Rect dimensions of the original
 #define EXTERN extern
 #endif
 
-//--------------------
-// These two guys must be declared manually in text_public.c !!!@
-//
-EXTERN int Number_Of_Item_Types;
-EXTERN itemspec* ItemMap;
-EXTERN item_bonus * PrefixList;
-EXTERN item_bonus * SuffixList;
 EXTERN float timeout_from_item_drop;
 EXTERN int use_open_gl;
 EXTERN int command_line_override_for_screen_resolution;
@@ -154,9 +113,6 @@ EXTERN char* character_descriptions [ MAX_PERSONS ] ;
 #ifdef HAVE_LIBGL
 EXTERN GLuint all_freedroid_textures [ MAX_AMOUNT_OF_TEXTURES_WE_WILL_USE ] ;
 EXTERN int next_texture_index_to_use ;
-#else
-// EXTERN int all_freedroid_textures [ MAX_AMOUNT_OF_TEXTURES_WE_WILL_USE ] ;
-// EXTERN int next_texture_index_to_use ;
 #endif
 
 EXTERN char previous_part_strings [ ALL_PART_GROUPS ] [ 50 ] ;
@@ -167,22 +123,13 @@ EXTERN point InventorySize;
 EXTERN iso_image MouseCursorImageList[ NUMBER_OF_MOUSE_CURSOR_PICTURES ];
 EXTERN iso_image SpellLevelButtonImageList[ NUMBER_OF_SKILL_PAGES ];
 
-EXTERN double Conveyor_Belt_Speed;
 EXTERN int Number_Of_Droids_On_Ship;
-EXTERN double Time_For_Each_Phase_Of_Door_Movement;
-EXTERN double Blast_Damage_Per_Second;
-EXTERN double Blast_Radius;
-EXTERN double Druid_Radius_X;
-EXTERN double Druid_Radius_Y;
 EXTERN float LevelDoorsNotMovedTime;
 EXTERN float LevelGunsNotFiredTime;
-EXTERN double collision_lose_energy_calibrator;
 EXTERN configuration_for_freedroid GameConfig;
-EXTERN float CurrentCombatScaleFactor;
-EXTERN int MaxMessageTime;
-EXTERN int MinMessageTime;
 EXTERN BFont_Info *Menu_BFont;
-EXTERN BFont_Info *Message_BFont;
+EXTERN BFont_Info *Messagevar_BFont;
+EXTERN BFont_Info *Messagestat_BFont;
 EXTERN BFont_Info *Para_BFont;
 EXTERN BFont_Info *FPS_Display_BFont;
 EXTERN BFont_Info *Blue_BFont;
@@ -191,9 +138,6 @@ EXTERN BFont_Info *Highscore_BFont;
 EXTERN float Overall_Average;
 EXTERN int SkipAFewFrames;
 
-EXTERN int global_ignore_doors_for_collisions_flag;
-EXTERN int global_check_for_light_only_collisions_flag;
-
 EXTERN int global_ingame_mode ;
 
 #undef EXTERN
@@ -204,13 +148,24 @@ EXTERN int global_ingame_mode ;
 #endif
 EXTERN iso_image light_radius_chunk[ NUMBER_OF_SHADOW_IMAGES ];
 EXTERN SDL_Rect InventoryRect;
-EXTERN SDL_Rect SkillScreenRect;
 EXTERN SDL_Rect CharacterRect;
 #ifdef HAVE_LIBGL
-EXTERN GLuint *automap_texture;  // this is to store an open_gl texture...
-EXTERN GLuint *light_radius_stretch_texture;  // this is to store an open_gl texture...
+EXTERN GLuint automap_texture;  // this is to store an open_gl texture...
+EXTERN GLuint light_radius_stretch_texture;  // this is to store an open_gl texture...
 EXTERN SDL_Surface* light_radius_stretch_surface ;
 #endif
+EXTERN int amask;
+EXTERN int gmask;
+EXTERN int bmask;
+EXTERN int rmask;
+
+#undef EXTERN
+#ifdef _light_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
+EXTERN light_radius_config LightRadiusConfig;
 
 #undef EXTERN
 #ifdef _sound_c
@@ -226,9 +181,6 @@ EXTERN SDL_Surface* light_radius_stretch_surface ;
 #define EXTERN extern
 #endif
 
-EXTERN int first_index_of_bot_on_level [ MAX_LEVELS ] ;
-EXTERN int last_index_of_bot_on_level [ MAX_LEVELS ] ;
-
 #undef EXTERN
 #ifdef _graphics_c
 #define EXTERN
@@ -239,7 +191,7 @@ EXTERN int Number_Of_Bullet_Types;
 EXTERN SDL_Surface *Screen;   
 EXTERN SDL_Surface* StoredMenuBackground [ 2 ] ;
 #ifdef HAVE_LIBGL
-EXTERN GLuint * StoredMenuBackgroundTex [ 2 ] ;
+EXTERN GLuint StoredMenuBackgroundTex [ 2 ] ;
 #endif
 EXTERN int current_mouse_cursor_shape;
 
@@ -275,14 +227,12 @@ EXTERN int droid_gethit_animation_speed_factor [ ENEMY_ROTATION_MODELS_AVAILABLE
 EXTERN int droid_death_animation_speed_factor [ ENEMY_ROTATION_MODELS_AVAILABLE ];
 EXTERN int droid_stand_animation_speed_factor [ ENEMY_ROTATION_MODELS_AVAILABLE ];
 
-EXTERN SDL_Surface *TuxWorkingCopy [ MAX_PLAYERS ] [ TUX_BREATHE_PHASES + TUX_SWING_PHASES + TUX_GOT_HIT_PHASES ] [ MAX_TUX_DIRECTIONS ] ;   // A pointer to the surfaces containing the tux
 EXTERN iso_image floor_iso_images [ ALL_ISOMETRIC_FLOOR_TILES ] ;   
 
 EXTERN float iso_floor_tile_width;
 EXTERN float iso_floor_tile_height;
 EXTERN float iso_floor_tile_width_over_two;
 EXTERN float iso_floor_tile_height_over_two;
-EXTERN const SDL_VideoInfo *vid_info;  // info about current video mode 
 EXTERN int vid_bpp; 		       // bits per pixel 
 
 #undef EXTERN
@@ -291,13 +241,6 @@ EXTERN int vid_bpp; 		       // bits per pixel
 #else
 #define EXTERN extern
 #endif
-EXTERN unsigned char *BannerPicture;
-EXTERN unsigned char *Enemypointer;
-EXTERN unsigned char *Influencepointer;
-EXTERN unsigned char *MapBlocks;
-EXTERN unsigned char *Digitpointer;
-EXTERN unsigned char *MenuItemPointer;
-EXTERN unsigned char *Robotptr;
 EXTERN obstacle_spec obstacle_map [ NUMBER_OF_OBSTACLE_TYPES ];
 
 #undef EXTERN
@@ -333,9 +276,6 @@ EXTERN obstacle* level_editor_marked_obstacle;
 #else
 #define EXTERN extern
 #endif
-EXTERN SDL_Joystick *joy;
-EXTERN int joy_num_axes; /* number of joystick axes */ 
-EXTERN int joy_sensitivity;
 EXTERN point input_axis;  /* joystick (and mouse) axis values */
 EXTERN int always_show_items_text;
 
@@ -346,5 +286,33 @@ EXTERN int always_show_items_text;
 #define EXTERN extern
 #endif
 
+#undef EXTERN
+#ifdef _skills_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
 
+EXTERN SDL_Rect SkillScreenRect;
+
+#undef EXTERN
+#ifdef _items_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
+
+
+EXTERN int Number_Of_Item_Types;
+EXTERN itemspec* ItemMap;
+EXTERN short int item_count_per_class[10];
+EXTERN item_bonus * PrefixList;
+EXTERN item_bonus * SuffixList;
+
+EXTERN int game_root_mode;
+enum {
+    ROOT_IS_UNKNOWN = 0,
+    ROOT_IS_GAME,
+    ROOT_IS_LVLEDIT,
+};
 #endif  // _global_h
