@@ -2,6 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -44,14 +45,15 @@
 #define LEVEL_END_STRING	"end_of_level"
 
 #define AREA_NAME_STRING "Area name=\""
-#define LEVEL_NAME_STRING "Name of this level="
+#define LEVEL_NAME_STRING "Name of this level=_\""
+#define LEVEL_NAME_STRING_LEGACY "Name of this level="
 #define LEVEL_ENTER_COMMENT_STRING "Infcmt=\""
 #define BACKGROUND_SONG_NAME_STRING "BgSong="
 #define MAP_END_STRING "/pmapinfolvl"
 
 #define ITEMS_SECTION_BEGIN_STRING "piteminfolvl"
 #define ITEMS_SECTION_END_STRING "/piteminfolvl"
-#define ITEM_CODE_STRING "it: t="
+#define ITEM_NAME_STRING "it: n=\""
 #define ITEM_POS_X_STRING " X="
 #define ITEM_POS_Y_STRING " Y="
 #define ITEM_AC_BONUS_STRING " AC="
@@ -96,6 +98,7 @@
 #define OBSTACLE_LABEL_BEGIN_STRING "pobstaclelabelinfolvl"
 #define OBSTACLE_LABEL_END_STRING "/pobstaclelabelinfolvl"
 #define OBSTACLE_LABEL_ANNOUNCE_STRING "obstacle_label_name=\""
+#define OBSTACLE_LABEL_STATES_STRING "states=\""
 #define INDEX_OF_OBSTACLE_NAME "obstacle_label.index="
 
 #define OBSTACLE_DESCRIPTION_BEGIN_STRING "pobsdescinfolvl"
