@@ -2,6 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -23,9 +24,9 @@
  *
  */
 
-/* ----------------------------------------------------------------------
+/**
  * This file contains all the console and lift functions (mostly)
- * ---------------------------------------------------------------------- */
+ */
 
 /*
  * This file has been checked for remains of german in the documentation.
@@ -43,159 +44,8 @@
 #include "proto.h"
 #include "SDL_rotozoom.h"
 
-enum 
-  {
-    NO_FUNCTION,
-    UNLOCK_FUNCTION,
-    GUNON_FUNCTION,
-    GUNOFF_FUNCTION
-  };
-
-int NoKeyPressed (void);
-int GreatItemShow ( int NumberOfItems , item* ShowPointerList[ MAX_ITEMS_IN_INVENTORY ] );
-
 void ShowDroidPicture (int PosX, int PosY, int Number );
 
-/* ----------------------------------------------------------------------
- * This function does the item show when the user has selected item
- * show from the console menu.
- * ---------------------------------------------------------------------- */
-int
-GreatItemShow ( int NumberOfItems , item* ShowPointerList[ MAX_ITEMS_IN_INVENTORY ] )
-{
-    int ItemType;
-    int Displacement=0;
-    bool finished = FALSE;
-    static int WasPressed = FALSE ;
-    // item* ShowPointerList[ MAX_ITEMS_IN_INVENTORY ];
-    // int NumberOfItems;
-    int ItemIndex=0;
-    int PasswordIndex = (-1) ;
-    // int ClearanceIndex = (-1) ;
-    int IdentifyAllowed = FALSE ;
-    char* MenuTexts[ 10 ];
-    MenuTexts[0]=_("Yes");
-    MenuTexts[1]=_("No");
-    MenuTexts[2]="";
-    
-    //--------------------
-    // We initialize the text rectangle
-    //
-    Cons_Text_Rect . x = 258 ;
-    Cons_Text_Rect . y = 89 ;
-    Cons_Text_Rect . w = 346 ;
-    Cons_Text_Rect . h = 282 ;
-    
-    // NumberOfItems = AssemblePointerListForItemShow ( &(ShowPointerList[0]), 0 );
-    
-    if ( ShowPointerList[0] == NULL )
-    {
-	MenuTexts[0]=_(" BACK ");
-	MenuTexts[1]="";
-	DoMenuSelection ( _(" YOU DONT HAVE ANYTHING IN INVENTORY, THAT COULD BE VIEWED. "), 
-			  MenuTexts, 1 , -1 , NULL );
-	return (-1) ;
-    }
-    
-    ItemType = ShowPointerList [ ItemIndex ] -> type ;
-    
-    Displacement = 0;
-    
-    while (!finished)
-    {
-	SDL_Delay (1);
-	
-	//--------------------
-	// We show all the info and the buttons that should be in this
-	// interface...
-	//
-	ShowItemInfo ( ShowPointerList [ ItemIndex ] , Displacement , TRUE , ITEM_BROWSER_BG_PIC_BACKGROUND_CODE , TRUE );
-	
-	// PutPasswordButtonsAndPassword ( PasswordIndex );
-	// PutSecurityButtonsAndClearance ( ClearanceIndex );
-	blit_our_own_mouse_cursor ( );
-	our_SDL_flip_wrapper( Screen );
-	
-	ItemType = ShowPointerList [ ItemIndex ] -> type ;
-	
-	if (SpacePressed() || EscapePressed() || MouseLeftPressed() )
-	{
-	    if ( MouseCursorIsOnButton( ITEM_BROWSER_RIGHT_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && MouseLeftPressed() && !WasPressed )
-	    {
-		if ( ItemIndex < NumberOfItems -1 ) 
-		{
-		    ItemIndex ++;	    
-		    MoveMenuPositionSound();
-		    Displacement = 0;
-		}
-	    }
-	    else if ( MouseCursorIsOnButton( ITEM_BROWSER_LEFT_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && MouseLeftPressed() && !WasPressed )
-	    {
-		if ( ItemIndex > 0) 
-		{
-		    ItemIndex --;	      
-		    MoveMenuPositionSound();
-		    Displacement = 0;
-		}
-	    }
-	    else if ( MouseCursorIsOnButton( UP_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && MouseLeftPressed() && !WasPressed )
-	    {
-		MoveMenuPositionSound();
-		Displacement += FontHeight ( GetCurrentFont () );
-	    }
-	    else if ( MouseCursorIsOnButton( DOWN_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && MouseLeftPressed() && !WasPressed )
-	    {
-		MoveMenuPositionSound();
-		// if (page > 0) page --;
-		Displacement -= FontHeight ( GetCurrentFont () );
-	    }
-	    else if ( MouseCursorIsOnButton( ITEM_BROWSER_EXIT_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  ) && MouseLeftPressed() && !WasPressed )
-	    {
-		finished = TRUE;
-		while (SpacePressed() ||EscapePressed());
-	    }
-	    
-	}
-	
-	WasPressed = MouseLeftPressed();
-	
-	if (UpPressed() || MouseWheelUpPressed())
-	{
-	    MoveMenuPositionSound();
-	    while (UpPressed());
-	    Displacement += FontHeight ( GetCurrentFont () );
-	}
-	if (DownPressed() || MouseWheelDownPressed())
-	{
-	    MoveMenuPositionSound();
-	    while (DownPressed());
-	    Displacement -= FontHeight ( GetCurrentFont () );
-	}
-	if (RightPressed() )
-	{
-	    MoveMenuPositionSound();
-	    while (RightPressed());
-	    if ( ItemType < Me[0].type) ItemType ++;
-	}
-	if (LeftPressed() )
-	{
-	    MoveMenuPositionSound();
-	    while (LeftPressed());
-	    if (ItemType > 0) ItemType --;
-	}
-	
-	if ( EscapePressed() )
-	{
-	    while ( EscapePressed() );
-	    return (-1);
-	}
-	
-    } // while !finished 
-    
-    return ( ItemIndex ) ;  // Currently equippment selection is not yet possible...
-    
-}; // int GreatItemShow ( int NumberOfItems , item* ShowPointerList[ MAX_ITEMS_IN_INVENTORY ] )
-
 /* ------------------------------------------------------------
  * display infopage page of droidtype
  * does update the screen, no our_SDL_flip_wrapper() necesary !
@@ -231,29 +81,19 @@ ShowDroidInfo ( int droidtype, int Displacement , char ShowArrows )
     PutString ( Screen , 330 * GameConfig . screen_width / 640 , 38 * GameConfig . screen_height / 480 , TextChunk );
     
     sprintf( InfoText, _("\
-Unit type %s - %s\n\
-Entry : %d\n\
-Class : %s\n\
-Drive : %s \n\
-Brain : %s"), Druidmap[droidtype].druidname, _(Classname[Druidmap[ droidtype ].class]),
-	     droidtype+1, _(Classes[Druidmap[droidtype].class]),
-	     ItemMap [ Druidmap[ droidtype ].drive_item.type ].item_name,
-	     _(Brainnames[ Druidmap[droidtype].brain ]));
+Unit type %s\n\
+Entry : %d\n"), Druidmap[droidtype].druidname, droidtype + 1);
     
     if ( (type = Druidmap[droidtype].weapon_item.type) >= 0) // make sure item=-1 
-	item_name = ItemMap[type].item_name;                 // does not segfault 
+	item_name = D_(ItemMap[type].item_name);                 // does not segfault 
     else 
-	item_name = "none";
+	item_name = _("none");
     
-    sprintf( TextChunk , _("\nArmamant : %s\n\
-Sensors  1: %s\n          2: %s\n          3: %s"),
-	     item_name,
-	     _(Sensornames[ Druidmap[droidtype].sensor1 ]),
-	     _(Sensornames[ Druidmap[droidtype].sensor2 ]),
-	     _(Sensornames[ Druidmap[droidtype].sensor3 ]));
+    sprintf( TextChunk , _("\nArmament : %s\n"),
+	     item_name);
     strcat ( InfoText , TextChunk );
     
-    sprintf ( TextChunk , _("\nNotes: %s\n"), Druidmap[droidtype].notes);
+    sprintf ( TextChunk , _("\nNotes: %s\n"), D_(Druidmap[droidtype].notes));
     strcat ( InfoText , TextChunk );
     
     SetCurrentFont( FPS_Display_BFont );
@@ -267,9 +107,9 @@ Sensors  1: %s\n          2: %s\n          3: %s"),
     
 }; // void ShowDroidInfo ( ... )
 
-/* ----------------------------------------------------------------------
+/**
  * This function displays an item picture. 
- * ---------------------------------------------------------------------- */
+ */
 void
 ShowItemPicture (int PosX, int PosY, int Number )
 {
@@ -381,7 +221,7 @@ ShowItemPicture (int PosX, int PosY, int Number )
 		if ( NumberOfImagesInThisRotation <= 0 )
 		{
 		    fprintf( stderr, "\n\nfpath: %s. \n" , fpath );
-		    GiveStandardErrorMessage ( __FUNCTION__  , "\
+		    ErrorMessage ( __FUNCTION__  , "\
 Freedroid was unable to load even one image of a rotated item image series into memory.\n\
 This error indicates some installation problem with freedroid.",
 					       PLEASE_INFORM, IS_FATAL );
@@ -398,7 +238,7 @@ This error indicates some installation problem with freedroid.",
 	    if ( i >= MAX_NUMBER_OF_IMAGES_IN_ITEM_ROTATION -2 )
 	    {
 		fprintf( stderr, "\n\nfpath: %s. \n" , fpath );
-		GiveStandardErrorMessage ( __FUNCTION__  , "\
+		ErrorMessage ( __FUNCTION__  , "\
 Freedroid was encountered more item images in an item rotation image series\n\
 than it is able to handle.  This is a very strange error.  Someone has been\n\
 trying to make the ultra-fine item rotation series.  Strange.",
@@ -406,8 +246,8 @@ trying to make the ultra-fine item rotation series.  Strange.",
 	    }
 	    
 	    SDL_SetAlpha( Whole_Image , 0 , SDL_ALPHA_OPAQUE );
-	    ItemRotationSurfaces[i] = our_SDL_display_format_wrapperAlpha( Whole_Image ); // now we have an alpha-surf of right size
-	    SDL_SetColorKey( ItemRotationSurfaces[i] , 0 , 0 ); // this should clear any color key in the dest surface
+	    ItemRotationSurfaces[i] = SDL_CreateRGBSurface ( 0, Whole_Image -> w, Whole_Image -> h, 32, rmask, gmask, bmask, amask);
+	    SDL_BlitSurface ( Whole_Image, NULL, ItemRotationSurfaces[i], NULL );
 	    SDL_FreeSurface( Whole_Image );
 	    
 	    // We must remember, that his is already loaded of course
@@ -432,9 +272,9 @@ trying to make the ultra-fine item rotation series.  Strange.",
 }; // void ShowItemPicture ( ... )
 
 
-/* ----------------------------------------------------------------------
+/**
  * This function displays an item picture. 
- * ---------------------------------------------------------------------- */
+ */
 void
 ShowDroidPicture (int PosX, int PosY, int Number )
 {
@@ -494,7 +334,7 @@ char fpath[2048];
 	    if ( Whole_Image == NULL )
 	    {
 		fprintf( stderr, "\n\nfpath: %s. \n" , fpath );
-		GiveStandardErrorMessage ( __FUNCTION__  , "\
+		ErrorMessage ( __FUNCTION__  , "\
 Freedroid was unable to load an image of a rotated droid into memory.\n\
 This error indicates some installation problem with freedroid.",
 					   PLEASE_INFORM, IS_FATAL );
@@ -523,10 +363,10 @@ This error indicates some installation problem with freedroid.",
     
 }; // void ShowDroidPicture ( ... )
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 write_full_item_name_into_string ( item* ShowItem , char* full_item_name ) 
 {
@@ -545,19 +385,9 @@ write_full_item_name_into_string ( item* ShowItem , char* full_item_name )
       strcat ( full_item_name , font_switchto_neon );
     }
 
-  if ( ShowItem->type == ITEM_MONEY ) sprintf( full_item_name , "%d " , ShowItem->multiplicity );
+  if ( MatchItemWithName(ShowItem->type, "Cyberbucks") ) sprintf( full_item_name , "%d " , ShowItem->multiplicity );
 
-  //--------------------
-  // If the item is is magical, we give the prefix name of course.
-  // In any case we'll give the suffix name and then, if the item
-  // is identified we'll also append any suffix to the description
-  // string.
-  //
-  if ( ( ShowItem->prefix_code != (-1) ) && ( ShowItem->is_identified ) )
-    strcat( full_item_name , PrefixList[ ShowItem->prefix_code ].bonus_name );
-  strcat( full_item_name , ItemMap[ ShowItem->type ].item_name );
-  if ( ( ShowItem->suffix_code != (-1) ) && ( ShowItem->is_identified ) )
-    strcat( full_item_name , SuffixList[ ShowItem->suffix_code ].bonus_name );
+  strcat( full_item_name , D_(ItemMap[ ShowItem->type ].item_name ));
 
   //--------------------
   // If the item is magical but not identified, we might add the word
@@ -583,11 +413,14 @@ write_full_item_name_into_string ( item* ShowItem , char* full_item_name )
 void 
 ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int background_code , int title_text_flag )
 {
-    char InfoText[10000];
+		static item LastItemShown;
+    static char InfoText[10000];
     char TextChunk[2000];
     char* ClassString;
     long int repairPrice = 0;
-    
+		
+		if( ShowItem == NULL) return;
+		
     SDL_SetClipRect ( Screen , NULL );
     
     blit_special_background ( background_code );
@@ -601,26 +434,27 @@ ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int backgro
     if ( title_text_flag )
     {
 	SetCurrentFont ( Menu_BFont );
-	strcpy ( TextChunk , ItemMap [ ShowItem->type ] . item_name );
+	strcpy ( TextChunk , D_(ItemMap [ ShowItem->type ] . item_name ));
 	CutDownStringToMaximalSize ( TextChunk , 225 );
 	PutString ( Screen , 330, 38, TextChunk );
     }
     
     //--------------------
     // Now we can display the rest of the smaller-font item description.
-    //
+    // If the item has changed, we must first assemble the description
+		//
+		if( memcmp( ShowItem, &LastItemShown, sizeof(item)))
+		{
     if ( ItemMap [ ShowItem->type ] . item_can_be_installed_in_weapon_slot )
 	ClassString = _("Weapon");
     else if ( ItemMap [ ShowItem->type ] . item_can_be_installed_in_drive_slot )
 	ClassString = _("Drive"); 
     else if ( ItemMap [ ShowItem->type ] . item_can_be_installed_in_armour_slot )
-	ClassString = _("Armour"); 
+	ClassString = _("Armor"); 
     else if ( ItemMap [ ShowItem->type ] . item_can_be_installed_in_shield_slot )
 	ClassString = _("Shield"); 
     else if ( ItemMap [ ShowItem->type ] . item_can_be_installed_in_special_slot )
 	ClassString = _("Helm"); 
-    else if ( ItemMap [ ShowItem->type ] . item_can_be_installed_in_aux_slot )
-	ClassString = _("Wristband/Collar"); 
     else ClassString = _("Miscellaneous"); 
     
     write_full_item_name_into_string ( ShowItem , TextChunk ) ;
@@ -648,7 +482,7 @@ ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int backgro
     
     if ( ItemMap [ ShowItem->type ] . item_group_together_in_inventory )
     {
-	if ( ShowItem -> type != ITEM_MONEY )
+	if ( ! MatchItemWithName(ShowItem -> type, "Cyberbucks") )
 		{
 		strcat ( InfoText , _("Multiplicity: "));
 		sprintf( TextChunk, "%d \n" , 
@@ -670,9 +504,9 @@ ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int backgro
 	{
 	strcat ( InfoText , _("Attributes required: "));
     
-	if ( ( ItemMap [ ShowItem->type ] . item_require_strength == (-1) ) &&
-	 ( ItemMap [ ShowItem->type ] . item_require_dexterity == (-1) ) &&
-	 ( ItemMap [ ShowItem->type ] . item_require_magic == (-1) ) )
+	if ( ( ItemMap [ ShowItem->type ] . item_require_strength <= 0 ) &&
+	 ( ItemMap [ ShowItem->type ] . item_require_dexterity <= 0 ) &&
+	 ( ItemMap [ ShowItem->type ] . item_require_magic <= 0) )
 	    {
 	    strcat ( InfoText , _("NONE\n"));
 	    }
@@ -747,6 +581,12 @@ ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int backgro
 	sprintf( TextChunk, _("Unsellable\n"));
         strcat ( InfoText , TextChunk );
 	}
+    
+    if ( ( ShowItem->suffix_code != (-1) ) || ( ShowItem->prefix_code != (-1) ) )
+    	{
+        if ( ShowItem->is_identified == FALSE )
+		strcat( InfoText, _("Identification cost: 100\n"));
+	}
   
     //--------------------
     // If the item is a weapon, then we print out some weapon stats...
@@ -781,24 +621,43 @@ ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int backgro
     }
     
     sprintf ( TextChunk, _("Notes: %s"), 
-	      ItemMap [ ShowItem->type ] . item_description );
+	      D_(ItemMap [ ShowItem->type ] . item_description) );
     strcat ( InfoText, TextChunk );
     
     switch ( ItemMap [ ShowItem->type ] . item_gun_use_ammunition )
     {
-	case ITEM_PLASMA_AMMUNITION:
+	case 2:
 	    strcat ( InfoText, _(" This weapon requires standard plasma ammunition."));
 	    break;
-	case ITEM_LASER_AMMUNITION:
+	case 1:
 	    strcat ( InfoText, _(" This weapon requires standard laser crystal ammunition."));
 	    break;
-	case ITEM_EXTERMINATOR_AMMUNITION:
+	case 3:
 	    strcat ( InfoText, _(" This weapon requires standard exterminator ammunition spheres."));
 	    break;
-	default:
+	case 4:
+	    strcat ( InfoText, _(" This weapon requires .22 Long Rifle rounds."));
+	    break;
+	case 5: 
+	    strcat ( InfoText, _(" This weapon requires Shotgun shells."));
+	    break; 
+	case 6: 
+            strcat ( InfoText, _(" This weapon requires 9x19mm rounds."));
+	    break;
+	case 7:
+            strcat ( InfoText, _(" This weapon requires 7.62x39mm rounds."));
+	    break;
+	case 8:
+            strcat ( InfoText, _(" This weapon requires .50 (12.7x99mm) Browning Machine Gun rounds."));
 	    break;
     }
-    
+		
+		//-----------------------
+		// We cache the item
+		// 
+		memcpy ( &LastItemShown, ShowItem, sizeof( item));
+		}
+		
     // SetCurrentFont( Para_BFont );
     // SetCurrentFont( Menu_BFont );
     SetCurrentFont( FPS_Display_BFont );
@@ -812,18 +671,4 @@ ShowItemInfo ( item* ShowItem , int Displacement , char ShowArrows , int backgro
     
 }; // void ShowItemInfo ( ... )
 
-/* ----------------------------------------------------------------------
- * This function fills the whole User_Rect with color 0 = black.
- * ---------------------------------------------------------------------- */
-void
-ClearUserFenster (void)
-{
-  SDL_Rect tmp;
-  
-  Copy_Rect (User_Rect, tmp)
-
-  our_SDL_fill_rect_wrapper( Screen , &tmp, 0 );
-
-}; // void ClearUserFenster( void )
-
 #undef _ship_c
