@@ -1,6 +1,7 @@
 /* 
  *
  *   Copyright (c) 2002, 2003 Johannes Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *
  *  This file is part of Freedroid
  *
@@ -21,11 +22,11 @@
  *
  */
 
-/* ----------------------------------------------------------------------
+/**
  * This file contains all the functions managing the character skills,
  * which means all the special functions and also the spells of the
  * players character.
- * ---------------------------------------------------------------------- */
+ */
 
 #define _skills_c
 
@@ -70,12 +71,10 @@
 
 int Override_Power_Limit=0;
 
-SDL_Rect SkillScreenRect;
-
-/* ----------------------------------------------------------------------
+/**
  * This function improves a generic skill (hack melee ranged magic) by one
  * 
- * ---------------------------------------------------------------------- */
+ */
 void
 ImproveSkill( int * skill )
 {
@@ -89,16 +88,49 @@ ImproveSkill( int * skill )
  * -----------------*/
 int calculate_program_heat_cost ( int program_id )
 {
-    float cost_ratio [ NUMBER_OF_SKILL_LEVELS ] = { 1.5, 1.3, 1.2, 1.0, 0.9, 0.8, 0.7, 0.6, 0.5, 0.4 };
-    return cost_ratio [ Me [ 0 ] . spellcasting_skill ]* ( SpellSkillMap[ program_id ] . heat_cost  + SpellSkillMap[ program_id ] . heat_cost_per_level * (Me [ 0 ] . SkillLevel [ program_id ] - 1 ));
+//                                               0.9^0, 0.9^1, 0.9^2 ... ...0.9^9
+    float cost_ratio [ NUMBER_OF_SKILL_LEVELS ] = { 1.0, 0.9, 0.81, 0.73, 0.66, 0.59, 0.53, 0.48, 0.43, 0.39 };
+    return cost_ratio [ Me . spellcasting_skill ]* ( SpellSkillMap[ program_id ] . heat_cost  + SpellSkillMap[ program_id ] . heat_cost_per_level * (Me . SkillLevel [ program_id ] - 1 ));
 };
 
 /* ------------------
  * This function calculates the damage dealt by a hit of a given program
  * -----------------*/
-int calculate_program_hit_damage ( int program_id )
+static int calculate_program_hit_damage ( int program_id )
 {
-    return ( SpellSkillMap[ program_id ] . damage_base  + SpellSkillMap[ program_id ] . damage_per_level * ( Me [ 0 ] . SkillLevel [ program_id ] - 1 ) + MyRandom(SpellSkillMap[ program_id ] . damage_mod));
+    return ( SpellSkillMap[ program_id ] . damage_base  + SpellSkillMap[ program_id ] . damage_per_level * ( Me . SkillLevel [ program_id ] - 1 ) + MyRandom(SpellSkillMap[ program_id ] . damage_mod));
+}
+
+/* ------------------
+ * This function calculates the lowest damage a program can deal
+ *-----------------*/
+static int calculate_program_hit_damage_low ( int program_id )
+{
+    return ( SpellSkillMap[ program_id ] . damage_base  + SpellSkillMap[ program_id ] . damage_per_level * ( Me . SkillLevel [ program_id ] - 1 ));
+
+}
+
+static int calculate_program_hit_damage_high ( int program_id )
+{
+    return ( SpellSkillMap[ program_id ] . damage_base  + SpellSkillMap[ program_id ] . damage_per_level * ( Me . SkillLevel [ program_id ] - 1 ) + SpellSkillMap[ program_id ] . damage_mod);
+}
+
+/* ------------------
+ * This function calculates the duration of the special effect of a
+ * given program
+ * ------------------*/
+static float calculate_program_effect_duration ( int program_id )
+{
+    return ( SpellSkillMap[ program_id ] . effect_duration  + SpellSkillMap[ program_id ] . effect_duration_per_level * ( Me . SkillLevel [ program_id ] - 1 ));
+}
+
+/**
+ * This function calculates the busy time (recovery time) for tux after running a program
+ */
+static float calculate_program_busy_time ( )
+{
+    float busy_time [ NUMBER_OF_SKILL_LEVELS ] = { 1.0, 0.9, 0.81, 0.73, 0.66, 0.59, 0.53, 0.48, 0.43, 0.39 };
+    return busy_time[ Me . spellcasting_skill ];
 }
 
 /* ------------------
@@ -116,59 +148,63 @@ int get_program_index_with_name ( const char * pname )
 
 	fprintf(stderr, "%s\n", pname);
 
-	 GiveStandardErrorMessage ( __FUNCTION__  , "\
+	 ErrorMessage ( __FUNCTION__  , "\
 FreedroidRPG could not find the program name above in the program spec array!",
                                  PLEASE_INFORM, IS_FATAL );
      return -1;
 }
 
-/* ----------------------------------------------------------------------
+/**
  * This function creates a teleporter portal to the home location.
- * ---------------------------------------------------------------------- */
+ */
 void
 TeleportHome ( void )
 {
 location HomeSpot;
 
-     if( (! Me [ 0 ] . teleport_anchor . x) && (! Me [ 0 ] . teleport_anchor . y)) //if there is no anchor, teleport home
+     if( (! Me . teleport_anchor . x) && (! Me . teleport_anchor . y)) //if there is no anchor, teleport home
         {
-                Me [ 0 ] . teleport_anchor . x = Me [ 0 ] . pos . x;
-                Me [ 0 ] . teleport_anchor . y = Me [ 0 ] . pos . y;
-                Me [ 0 ] . teleport_anchor . z = Me [ 0 ] . pos . z;
+                Me . teleport_anchor . x = Me . pos . x;
+                Me . teleport_anchor . y = Me . pos . y;
+                Me . teleport_anchor . z = Me . pos . z;
                 teleport_arrival_sound ( );
                 ResolveMapLabelOnShip ( "TeleportHomeTarget" , &(HomeSpot) );
-                Teleport ( HomeSpot.level , HomeSpot.x , HomeSpot.y , 0 , TRUE ) ;
+                Teleport ( HomeSpot.level , HomeSpot.x , HomeSpot.y , TRUE ) ;
         }
         else //we must teleport back to the anchor
         {
                 teleport_arrival_sound  ( );
-                Teleport ( Me [ 0 ] . teleport_anchor . z , Me [ 0 ] . teleport_anchor . x , Me [ 0 ] . teleport_anchor . y , 0 , TRUE ) ;
+                Teleport ( Me . teleport_anchor . z , Me . teleport_anchor . x , Me . teleport_anchor . y , TRUE ) ;
+		Me . teleport_anchor . x = 0;
+		Me . teleport_anchor . y = 0;
+		Me . teleport_anchor . z = 0;
         }
 
 }; // void TeleportHome ( void )
 
-/* ----------------------------------------------------------------------
- * This function handles the skills the player might have acitivated
- * or deactivated or whatever.
- * ---------------------------------------------------------------------- */
+/**
+ * This function handles the program the player has just activated.
+ * It checks temperature (does not increase it), and makes sure a 
+ * busy_time is set.
+ */
 void
 HandleCurrentlyActivatedSkill()
 {
-    if ( ! MouseRightClicked() ) 
+    if ( ! MouseRightClicked() || Me . busy_time > 0 ) 
 	return;
     
     int Grabbed_InvPos = GetInventoryItemAt ( GetInventorySquare_x ( GetMousePos_x() ) , GetInventorySquare_y ( GetMousePos_y() ) );
     if ( Grabbed_InvPos != -1 )
-	if ( ItemMap [ Me[0].Inventory[ Grabbed_InvPos ] . type ] . item_can_be_applied_in_combat ) 
+	if ( ItemMap [ Me.Inventory[ Grabbed_InvPos ] . type ] . item_can_be_applied_in_combat ) 
 	return; //if the cursor is over an item that can be applied, then the player wants to apply it not trigger a spell
     
-    if ( Me [ 0 ] . SkillLevel [ Me [ 0 ] . readied_skill ] <= 0 ) return;
+    if ( Me . SkillLevel [ Me . readied_skill ] <= 0 ) return;
 
     /* We calculate the spellcost and check the power limit override - the temperature is raised further down, when the actual effect
 	gets triggered */
-    int SpellCost = calculate_program_heat_cost ( Me [ 0 ] . readied_skill );
+    int SpellCost = calculate_program_heat_cost ( Me . readied_skill );
 
-    if ( Me [ 0 ] . temperature > Me [ 0 ] . max_temperature - SpellCost && !Override_Power_Limit )
+    if ( Me . temperature > Me . max_temperature - SpellCost && !Override_Power_Limit )
 	{
 	      //Not_Enough_Mana_Sound(  );
 	      Override_Power_Limit = 1;
@@ -176,7 +212,7 @@ HandleCurrentlyActivatedSkill()
 	}
     Override_Power_Limit = 0;
 
-    switch ( SpellSkillMap [ Me [ 0 ] . readied_skill ] . form ) 
+    switch ( SpellSkillMap [ Me . readied_skill ] . form ) 
 	{
 	case PROGRAM_FORM_IMMEDIATE:
 	case PROGRAM_FORM_BULLET:
@@ -184,7 +220,17 @@ HandleCurrentlyActivatedSkill()
 		if (! MouseCursorIsInUserRect ( GetMousePos_x() , GetMousePos_y() ) ) return;; break;
 	}
 
-    DoSkill(Me [ 0 ] . readied_skill, SpellCost);
+    DoSkill(Me . readied_skill, SpellCost);
+
+    /* Certain special actions implemented through DoSkill may set their own
+     * busy time, such as weapon reload. In that case, do not touch busy_time.
+     * Otherwise, mark that we are running a program.
+     */
+
+    if (Me.busy_time == 0) {
+	Me . busy_time = calculate_program_busy_time();
+	Me . busy_type = RUNNING_PROGRAM;
+    }
 
 return;    
 }; // void HandleCurrentlyActivatedSkill( void )
@@ -192,60 +238,92 @@ return;
 int 
 DoSkill(int skill_index, int SpellCost)
 {
-    int index_of_droid_below_mouse_cursor = ( -1 ) ;
-    int player_num = 0;
+    enemy * droid_below_mouse_cursor = NULL;
+
+    float hitdmg = calculate_program_hit_damage ( skill_index );
+    float effdur = calculate_program_effect_duration ( skill_index );
 
     /*we handle the form of the program now*/
     switch ( SpellSkillMap [ skill_index ] . form ) 
 	{
 	case PROGRAM_FORM_IMMEDIATE:
-		index_of_droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor ( 0 ) ;
-		if ( index_of_droid_below_mouse_cursor == ( -1 ) ) 
+		droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor ( ) ;
+		if ( droid_below_mouse_cursor == NULL ) 
 			goto done_handling_instant_hits;
-		if ( ! DirectLineWalkable ( Me [ player_num ] . pos . x , Me [ player_num ] . pos . y ,  translate_pixel_to_map_location ( 0 ,
-                                                       (float) ServerThinksInputAxisX ( 0 ) ,
-                                                       (float) ServerThinksInputAxisY ( 0 ) , TRUE ), 
-							translate_pixel_to_map_location ( 0 ,
-                                                       (float) ServerThinksInputAxisX ( 0 ) ,
-                                                       (float) ServerThinksInputAxisY ( 0 ) , FALSE ), Me [ 0 ] . pos . z))
-						goto done_handling_instant_hits;
+		if ( ! DirectLineColldet ( Me . pos . x ,
+					    Me . pos . y ,
+					    translate_pixel_to_map_location ( (float) input_axis.x ,
+									      (float) input_axis.y ,
+									      TRUE ) ,
+					    translate_pixel_to_map_location ( (float) input_axis.x ,
+									      (float) input_axis.y ,
+									      FALSE ) ,
+					    Me . pos . z,
+					    &FilterFlyable))
+		    goto done_handling_instant_hits;
 
-		AllEnemys [ index_of_droid_below_mouse_cursor ] . energy -= calculate_program_hit_damage ( skill_index ) ;
-	        Me [ 0 ] . temperature += SpellCost;
+		if (( Druidmap [ droid_below_mouse_cursor->type ] . is_human && ! SpellSkillMap [ skill_index ]  . hurt_humans )
+		   || ( ! Druidmap [ droid_below_mouse_cursor->type ] . is_human && ! SpellSkillMap [ skill_index ]  . hurt_bots ))
+		    return 1;
+
+		if ( hitdmg > 0 )
+		    hit_enemy(droid_below_mouse_cursor, hitdmg, 1, -1, 1);
+		
+		if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) )
+		    droid_below_mouse_cursor -> paralysation_duration_left += effdur;
+		if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "slowdown" ) )
+		    droid_below_mouse_cursor -> frozen += effdur;
+		if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" )  )
+		    {
+		    droid_below_mouse_cursor -> poison_duration_left += effdur;
+		    droid_below_mouse_cursor -> poison_damage_per_sec += hitdmg;
+		    }
+
+
+	        Me . temperature += SpellCost;
 		break;
 	
 	case PROGRAM_FORM_SELF:
-		Me [ 0 ] . energy -= calculate_program_hit_damage ( skill_index ) ;
-                Me [ 0 ] . temperature += SpellCost;
-		Me [ 0 ] . slowdown_duration += strcmp( SpellSkillMap [ skill_index ] . effect, "slowdown" ) ? 0 : 10;
-                Me [ 0 ] . paralyze_duration += strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) ? 0 : 15;
+		Me . energy -= hitdmg;
+                Me . temperature += SpellCost;
+		Me . slowdown_duration += strcmp( SpellSkillMap [ skill_index ] . effect, "slowdown" ) ? 0 : effdur;
+                Me . paralyze_duration += strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) ? 0 : effdur;
+		Me . invisible_duration += strcmp ( SpellSkillMap [ skill_index ] . effect, "invisibility" ) ? 0 : effdur;
 		break;
 
 	case PROGRAM_FORM_BULLET:
-                Me [ 0 ] . temperature += SpellCost;
+                Me . temperature += SpellCost;
 	
 		moderately_finepoint target_location;
-	        target_location . x = translate_pixel_to_map_location ( 0 , ServerThinksInputAxisX ( 0 ) , ServerThinksInputAxisY ( 0 ) , TRUE ) ;
-		target_location . y = translate_pixel_to_map_location ( 0 , ServerThinksInputAxisX ( 0 ) , ServerThinksInputAxisY ( 0 ) , FALSE ) ;
+	        target_location . x = translate_pixel_to_map_location ( input_axis.x , input_axis.y , TRUE ) ;
+		target_location . y = translate_pixel_to_map_location ( input_axis.x , input_axis.y , FALSE ) ;
 		
 		bullet bul_parms;
+		/*XXX hardcoded laser pistol type*/
 		if (  SpellSkillMap [ skill_index ] . graphics_code != -1 ) 
-			FillInDefaultBulletStruct( 0, &bul_parms,  SpellSkillMap [ skill_index ] . graphics_code, ITEM_LASER_PISTOL );
-		else	FillInDefaultBulletStruct( 0, &bul_parms, MAGENTA_BULLET, ITEM_LASER_PISTOL );
+			FillInDefaultBulletStruct( &bul_parms,  SpellSkillMap [ skill_index ] . graphics_code, GetItemIndexByName("Laser pistol") );
+		else
+			FillInDefaultBulletStruct( &bul_parms, MAGENTA_BULLET, GetItemIndexByName("Laser pistol"));
 	
-		bul_parms.freezing_level = strcmp( SpellSkillMap [ skill_index ] . effect, "slowdown" ) ? 0 : 10;
-		bul_parms.poison_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : 10;
-		bul_parms.poison_damage_per_sec = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : 1;
-		bul_parms.paralysation_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) ? 0 : 10;
-		bul_parms.damage = calculate_program_hit_damage ( skill_index ) ;
-		bul_parms.to_hit = SpellHitPercentageTable [ Me [ 0 ] . spellcasting_skill ];
+		bul_parms.freezing_level = strcmp( SpellSkillMap [ skill_index ] . effect, "slowdown" ) ? 0 : effdur;
+		bul_parms.poison_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : effdur;
+		bul_parms.poison_damage_per_sec = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : hitdmg;
+		bul_parms.paralysation_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) ? 0 : effdur;
+		bul_parms.damage = hitdmg;
+		bul_parms.to_hit = SpellHitPercentageTable [ Me . spellcasting_skill ];
+		if (SpellSkillMap [ skill_index ]  . hurt_humans &&  SpellSkillMap [ skill_index ]  . hurt_bots)
+			bul_parms.hit_type = ATTACK_HIT_ALL;
+		else if (SpellSkillMap [ skill_index ]  . hurt_humans )
+		    	bul_parms.hit_type = ATTACK_HIT_HUMANS;
+		else bul_parms.hit_type = ATTACK_HIT_BOTS;
 
-		FireTuxRangedWeaponRaw ( 0 , ITEM_LASER_PISTOL , -1 , &bul_parms, target_location); 
+
+		FireTuxRangedWeaponRaw ( GetItemIndexByName("Laser pistol") , -1 , &bul_parms, target_location); 
 		
 		return 1; //no extra effects
 
 	case PROGRAM_FORM_RADIAL:
-		Me [ 0 ] . temperature += SpellCost;
+		Me . temperature += SpellCost;
 		
 		int i,j;
 		for ( i = 0 ; i < MAX_ACTIVE_SPELLS ; i ++ )
@@ -255,22 +333,27 @@ DoSkill(int skill_index, int SpellCost)
 	        if ( i >= MAX_ACTIVE_SPELLS ) i = 0 ;
 
 		AllActiveSpells [ i ] . img_type = (SpellSkillMap [ skill_index ] . graphics_code == -1 ? 2 : SpellSkillMap [ skill_index ] . graphics_code);
-		AllActiveSpells [ i ] . spell_center . x = Me [ 0 ] . pos . x;
-	        AllActiveSpells [ i ] . spell_center . y = Me [ 0 ] . pos . y;
+		AllActiveSpells [ i ] . spell_center . x = Me . pos . x;
+	        AllActiveSpells [ i ] . spell_center . y = Me . pos . y;
 	        AllActiveSpells [ i ] . spell_radius = 0.3 ;
 	        AllActiveSpells [ i ] . spell_age = 0 ;
 	        AllActiveSpells [ i ] . mine = 1;
+		if (SpellSkillMap [ skill_index ]  . hurt_humans &&  SpellSkillMap [ skill_index ]  . hurt_bots)
+		    AllActiveSpells [ i ].hit_type = ATTACK_HIT_ALL;
+		else if (SpellSkillMap [ skill_index ]  . hurt_humans )
+		    AllActiveSpells [ i ].hit_type = ATTACK_HIT_HUMANS;
+		else AllActiveSpells [ i ].hit_type = ATTACK_HIT_BOTS;
  
 	        for ( j = 0 ; j < RADIAL_SPELL_DIRECTIONS ; j ++ )
 	            { 
   	            AllActiveSpells [ i ] . active_directions [ j ] = TRUE ;
                     }
 		
-		AllActiveSpells [ i ] . freeze_duration = strcmp( SpellSkillMap [ skill_index ] . effect, "slowdown" ) ? 0 : 10;
-		AllActiveSpells [ i ] .poison_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : 10;
-		AllActiveSpells [ i ] .poison_dmg = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : 1;
-		AllActiveSpells [ i ] .paralyze_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) ? 0 : 10;
-		AllActiveSpells [ i ] .damage = calculate_program_hit_damage ( skill_index ) ;
+		AllActiveSpells [ i ] . freeze_duration = strcmp( SpellSkillMap [ skill_index ] . effect, "slowdown" ) ? 0 : effdur;
+		AllActiveSpells [ i ] .poison_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : effdur;
+		AllActiveSpells [ i ] .poison_dmg = strcmp ( SpellSkillMap [ skill_index ] . effect, "poison" ) ? 0 : hitdmg;
+		AllActiveSpells [ i ] .paralyze_duration = strcmp ( SpellSkillMap [ skill_index ] . effect, "paralyze" ) ? 0 : effdur ;
+		AllActiveSpells [ i ] .damage = hitdmg ;
 
 		return 1;
 
@@ -286,30 +369,40 @@ DoSkill(int skill_index, int SpellCost)
 	}
 
 
-    if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "talk_or_takeover" ) )
+    if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "takeover" ) )
    	{
 	if (! MouseCursorIsInUserRect ( GetMousePos_x() , GetMousePos_y() ) ) goto out;
-	index_of_droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor ( player_num ) ;
-	if ( index_of_droid_below_mouse_cursor == ( -1 ) ) goto out;
-	if ( ! DirectLineWalkable ( Me [ player_num ] . pos . x , Me [ player_num ] . pos . y ,  translate_pixel_to_map_location ( player_num ,
-                                                       (float) ServerThinksInputAxisX ( player_num ) ,
-                                                       (float) ServerThinksInputAxisY ( player_num ) , TRUE ), 
-							translate_pixel_to_map_location ( player_num ,
-                                                       (float) ServerThinksInputAxisX ( player_num ) ,
-                                                       (float) ServerThinksInputAxisY ( player_num ) , FALSE ), Me [ player_num ] . pos . z))
+	droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor ( ) ;
+	if ( droid_below_mouse_cursor == NULL ) goto out;
+	if ( ! DirectLineColldet ( Me . pos . x,
+				    Me . pos . y,
+				    translate_pixel_to_map_location ( (float) input_axis.x ,
+								      (float) input_axis.y , TRUE ),
+				    translate_pixel_to_map_location ( (float) input_axis.x ,
+								      (float) input_axis.y , FALSE ),
+				    Me . pos . z, NULL))
 		goto out;
 
 
-	if ( AllEnemys [ index_of_droid_below_mouse_cursor ] . is_friendly )
-		ChatWithFriendlyDroid( & ( AllEnemys [ index_of_droid_below_mouse_cursor ] ) );
-	else
+	if ( ! droid_below_mouse_cursor-> is_friendly ) 
 	    {
 	    //--------------------
 	    // Only droids can be hacked.  Humans can't be 
 	    // hacked.
 	    //
-	    if ( ! ( Druidmap [ AllEnemys [ index_of_droid_below_mouse_cursor ] . type ] . is_human ) )
-	        Takeover ( index_of_droid_below_mouse_cursor ) ;
+	    if ( Takeover ( droid_below_mouse_cursor ) )
+		    {
+		    // upon successful takeover
+                    // restore original heat
+		    // Me . temperature -= SpellCost;
+                    // go directly to chat to choose droid program
+                    if ( GameConfig . talk_to_bots_after_takeover )
+			    ChatWithFriendlyDroid ( droid_below_mouse_cursor ) ;     
+		    }
+	    }
+	else 
+	    {
+	    Me . temperature -= SpellCost; //pretend nothing happened
 	    }
 	goto out;
 	}
@@ -319,7 +412,7 @@ DoSkill(int skill_index, int SpellCost)
 	if (! MouseCursorIsInUserRect ( GetMousePos_x() , GetMousePos_y() ) ) goto out;
         if ( MouseCursorIsInUserRect ( GetMousePos_x()  , 
 				      GetMousePos_y()  ) )
-		tux_wants_to_attack_now ( 0 , TRUE );
+		tux_wants_to_attack_now ( TRUE );
 	goto out;
 	}
 
@@ -347,7 +440,7 @@ DoSkill(int skill_index, int SpellCost)
 	// click must be ignored completely.
 	//
 	if ( global_ingame_mode == GLOBAL_INGAME_MODE_IDENTIFY )
-		Me [ 0 ] . temperature -= SpellCost;
+		Me . temperature -= SpellCost;
 
         silently_unhold_all_items ( );
         global_ingame_mode = GLOBAL_INGAME_MODE_IDENTIFY ;
@@ -367,26 +460,21 @@ DoSkill(int skill_index, int SpellCost)
 
     if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "teleport_home" ) )
    	{
-               TeleportHome (  ) ;
+        TeleportHome (  ) ;
 	goto out;
 	}
 
 
-
-/*    if ( ! strcmp ( SpellSkillMap [ skill_index ] . effect, "" ) )
-   	{
-	}*/
-
 out:
 
 return 1;    
 }; // void HandleCurrentlyActivatedSkill( void )
 
 
-/* ----------------------------------------------------------------------
+/**
  * This function checks if a given screen position lies within the 
  * one of the skill icons and returns the number of that skill icon.
- * ---------------------------------------------------------------------- */
+ */
 int
 CursorIsOnWhichSkillButton( int x , int y )
 {
@@ -417,18 +505,15 @@ CursorIsOnWhichSkillButton( int x , int y )
     if ( y < SkillScreenRect.y + 16 + 4 * 64 + 4 * 16 ) return ( -1 );
     if ( y < SkillScreenRect.y + 16 + 5 * 64 + 4 * 16 ) return (  4 );
     
-     if ( y < SkillScreenRect.y + 16 + 5 * 64 + 5 * 16 ) return ( -1 );
-     if ( y < SkillScreenRect.y + 16 + 6 * 64 + 5 * 16 ) return (  5 );
-    
     return ( -1 );
 }; // int CursorIsOnWhichSkillButton( int x , int y )
 
 
-/* ----------------------------------------------------------------------
+/**
  * This function checks if a given screen position lies within 
  * one of the spell level buttons and returns the number of that 
  * spell level button.
- * ---------------------------------------------------------------------- */
+ */
 int
 CursorIsOnWhichSpellPageButton( int x , int y )
 {
@@ -457,13 +542,16 @@ CursorIsOnWhichSpellPageButton( int x , int y )
     return ( -1 );
 }; // int CursorIsOnWhichSpellLevelButton( int x , int y )
 
-/* ---------------------------------------------------------------------- 
+/** 
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 ShowSkillsExplanationScreen( void )
 {
+    int ICON_OFFSET_X = 20;
+    int ICON_OFFSET_Y = 20;
+    int TEXT_OFFSET_X = 15;
     SDL_Rect TargetSkillRect;
     
     //--------------------
@@ -472,37 +560,38 @@ ShowSkillsExplanationScreen( void )
     blit_special_background ( SKILL_SCREEN_EXPLANATION_BACKGROUND_CODE ) ;
 
     //--------------------
-    // We will use the FPS display font, cause the small one isn't 
-    // very well readable on the silver background
+    // Draws the skill icon at the correct position
     //
-    SetCurrentFont ( FPS_Display_BFont );
-    
-    TargetSkillRect.x = 15;
-    TargetSkillRect.y = 15;
-    
-    LoadOneSkillSurfaceIfNotYetLoaded ( Me [ 0 ] . readied_skill );
-    
+    TargetSkillRect.x = ICON_OFFSET_X;
+    TargetSkillRect.y = ICON_OFFSET_Y;
+    LoadOneSkillSurfaceIfNotYetLoaded ( Me . readied_skill );
     if ( use_open_gl )
     {
-	blit_open_gl_texture_to_screen_position ( &SpellSkillMap [ Me [ 0 ] . readied_skill ] . icon_surface , 
-						  TargetSkillRect . x , TargetSkillRect . y , TRUE );
+	draw_gl_textured_quad_at_screen_position ( &SpellSkillMap [ Me . readied_skill ] . icon_surface , 
+						  TargetSkillRect . x , TargetSkillRect . y );
     }
     else
     {
-	our_SDL_blit_surface_wrapper ( SpellSkillMap [ Me [ 0 ] . readied_skill ] . icon_surface . surface , 
+	our_SDL_blit_surface_wrapper ( SpellSkillMap [ Me . readied_skill ] . icon_surface . surface , 
 				       NULL , Screen , &TargetSkillRect );
     }
-    
-    
-    TargetSkillRect.w = 320 - 15 - 15;
-    TargetSkillRect.h = 480 - 15 ;
-    DisplayText( SpellSkillMap [ Me [ 0 ] . readied_skill ] . description , 16 , 16+64+16, 
+
+    //--------------------
+    // Draws the explanation text
+    // (We will use the FPS display font, cause the small one isn't 
+    // very well readable on the silver background.)
+    //
+    TargetSkillRect.x = TEXT_OFFSET_X;
+    TargetSkillRect.w = 320 - 2 * TEXT_OFFSET_X;
+    TargetSkillRect.h = 480 - 15;
+    SetCurrentFont ( FPS_Display_BFont );
+    DisplayText( D_(SpellSkillMap [ Me . readied_skill ] . description) , 16 , 16+64+16, 
 		 &TargetSkillRect , TEXT_STRETCH );
 
 
 }; // void ShowSkillsExplanationScreen( void )
 
-/* ---------------------------------------------------------------------- 
+/** 
  * We will draw only those skills to the skills inventory, that are
  * already present in the Tux.  That way the game remains open for new
  * skills to the player and he doesn't now in advance which skills there
@@ -513,7 +602,7 @@ ShowSkillsExplanationScreen( void )
  * The first few entries will be filled with internal skill index numbers
  * for reference.
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 establish_skill_subset_map ( int *SkillSubsetMap )
 {
@@ -525,7 +614,7 @@ establish_skill_subset_map ( int *SkillSubsetMap )
     }
   for ( i = 0 ; i < number_of_skills ; i ++ )
     {
-      if ( Me [ 0 ] . SkillLevel [ i ] > 0 )
+      if ( Me . SkillLevel [ i ] > 0 )
 	{
 	  SkillSubsetMap [ NextPosition ] = i ;
 	  NextPosition++;
@@ -533,10 +622,10 @@ establish_skill_subset_map ( int *SkillSubsetMap )
     }
 }; // void establish_skill_subset_map ( int *SkillSubsetMap );
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 activate_nth_aquired_skill ( int skill_num )
 {
@@ -555,25 +644,25 @@ activate_nth_aquired_skill ( int skill_num )
   //
   if ( SkillSubsetMap [ skill_num ] != (-1) )
     {
-      Me [ 0 ] . readied_skill = SkillSubsetMap [ skill_num ] ;
+      Me . readied_skill = SkillSubsetMap [ skill_num ] ;
     }
   else
     {
       for ( i = 0 ; i < number_of_skills ; i ++ )
 	{
 	  if ( SkillSubsetMap [ i ] != (-1) )
-	    Me [ 0 ] . readied_skill = SkillSubsetMap [ i ] ;
+	    Me . readied_skill = SkillSubsetMap [ i ] ;
 	}
     }
   
 }; // void activate_nth_skill ( int skill_num )
 
-/* ----------------------------------------------------------------------
+/**
  * This function displays the SKILLS SCREEN.  This is NOT the same as the
  * CHARACTER SCREEN.  In the skills screen you can see what skills/spells
  * you currenlty have availabe and you can select a new readied skill by
  * clicking on it with the mouse.
- * ---------------------------------------------------------------------- */
+ */
 void 
 ShowSkillsScreen ( void )
 {
@@ -679,9 +768,9 @@ ShowSkillsScreen ( void )
 	
 	if ( use_open_gl )
 	{
-	    blit_open_gl_texture_to_screen_position ( 
+	    draw_gl_textured_quad_at_screen_position ( 
 		&SpellSkillMap [ SkillOfThisSlot ] . icon_surface , 
-		ButtonRect . x , ButtonRect . y , TRUE );
+		ButtonRect . x , ButtonRect . y );
 	}
 	else
 	{
@@ -694,24 +783,104 @@ ShowSkillsScreen ( void )
 	// First we write the name of the skill to the screen
 	//
     //SetCurrentFont ( Menu_BFont );
-    SetCurrentFont ( FPS_Display_BFont );
+	SetCurrentFont ( FPS_Display_BFont );
     
-	DisplayText( SpellSkillMap [ SkillOfThisSlot ] . name , 
+	DisplayText( D_(SpellSkillMap [ SkillOfThisSlot ] . name) , 
 		     16 + 64 + 16 + SkillScreenRect.x , 
-		     FIRST_SKILLRECT_Y + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y , 
+		     FIRST_SKILLRECT_Y - 6 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y , 
 		     &SkillScreenRect , TEXT_STRETCH );
-      
-    SetCurrentFont ( Message_BFont );
-	//--------------------
-	// Now we write the competence of the players character in that skill to the screen
-	//
-	sprintf( CharText , _("Program revision: %d "), Me[0].SkillLevel[ SkillOfThisSlot ] );
-	DisplayText( CharText , 16 + 64 + 16 + SkillScreenRect.x , 
-		     FIRST_SKILLRECT_Y + i * ( 64 + INTER_SKILLRECT_DIST ) + SkillScreenRect.y + 2 * FontHeight( GetCurrentFont() ) , &SkillScreenRect , TEXT_STRETCH );
-	sprintf( CharText , _("Heat produced: %d "),   calculate_program_heat_cost ( SkillOfThisSlot )  );
-	DisplayText( CharText , 16 + 64 + 16 + SkillScreenRect.x , 
-		     FIRST_SKILLRECT_Y + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y + 3 * FontHeight( GetCurrentFont() ) , &SkillScreenRect , TEXT_STRETCH );
+
+	SetCurrentFont ( Messagestat_BFont );
+	int tmp, tmp2;
+	int nextypos = FIRST_SKILLRECT_Y - 8 + i * ( 64 + INTER_SKILLRECT_DIST ) + SkillScreenRect.y + 2 * FontHeight( GetCurrentFont() );
 	
+	// Program revision
+	sprintf( CharText , _("Program revision: %c%d%c "), font_switchto_msgvar[0] ,  Me.SkillLevel[ SkillOfThisSlot ] , font_switchto_msgstat[0] );
+	DisplayText( CharText , 16 + 64 + 16 + SkillScreenRect.x , 
+		nextypos , &SkillScreenRect , TEXT_STRETCH );
+	nextypos += FontHeight( GetCurrentFont() );
+
+
+	// Heat cost/cooling
+	tmp = calculate_program_heat_cost ( SkillOfThisSlot );
+	if ( tmp != 0 )
+	    {
+	    if ( tmp > 0 )
+                sprintf( CharText , _("Heat produced: %c%d%c "),  font_switchto_msgvar[0] , tmp , font_switchto_msgstat[0] );
+	    else 
+		sprintf( CharText , _("Cooling: %c%d%c "),  font_switchto_msgvar[0] , -tmp , font_switchto_msgstat[0] );
+	    DisplayText( CharText , 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect , TEXT_STRETCH );
+	    nextypos += FontHeight( GetCurrentFont() );
+	    }
+
+	// Damage/healing
+	tmp = calculate_program_hit_damage_low( SkillOfThisSlot );
+	tmp2 = calculate_program_hit_damage_high ( SkillOfThisSlot );
+	if ( tmp != 0 )
+	    {
+	    if ( tmp > 0 )
+		{
+                if ( tmp == tmp2 )
+        		sprintf( CharText , _("Damage: %c%d%c "),  font_switchto_msgvar[0] , tmp , font_switchto_msgstat[0] );
+                else
+                        sprintf( CharText , _("Damage: %c%d-%d%c "),  font_switchto_msgvar[0] , tmp , tmp2 , font_switchto_msgstat[0] );
+		}
+	    else
+		{
+                if ( tmp == tmp2 )
+        		sprintf ( CharText, _("Healing: %c%d%c "),  font_switchto_msgvar[0] , -tmp , font_switchto_msgstat[0] );
+                else
+                        sprintf ( CharText, _("Healing: %c%d-%d%c "),  font_switchto_msgvar[0] , -tmp , -tmp2 , font_switchto_msgstat[0] );
+		}
+	    DisplayText( CharText , 16 + 64 + 16 + SkillScreenRect.x , nextypos , &SkillScreenRect , TEXT_STRETCH );
+	    nextypos += FontHeight( GetCurrentFont() );
+	    }
+
+	// Special effect and duration
+	if ( strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "none" ) )
+	    { 
+	    if ( ! strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "paralyze" ) )
+	       sprintf(CharText, _("Paralyze"));
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "slowdown" ))  
+		    sprintf ( CharText, _("Slow down"));
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "invisibility" ))
+		sprintf ( CharText, _("Invisible"));
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "poison" ))
+		sprintf ( CharText, _("Poison"));
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "takeover" ))
+                {
+                        tmp = Me.SkillLevel [ SkillOfThisSlot ] + 2;
+                        sprintf ( CharText, _("Takeover charges: %c%d%c "),  font_switchto_msgvar[0] , tmp , font_switchto_msgstat[0] );
+                }
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "teleport_home" ))
+		sprintf ( CharText, _("Escape"));
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "passive" ))
+		sprintf ( CharText, _("Passive"));
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "identify" ))
+		sprintf ( CharText, " ");
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "weapon" ))
+		sprintf ( CharText, " ");
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "repair" ))
+		sprintf ( CharText, " ");
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "satellite" ))
+		sprintf ( CharText, " ");
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "light" ))
+		sprintf ( CharText, " ");
+	    else if ( !strcmp ( SpellSkillMap [ SkillOfThisSlot ] . effect, "burnup" ))
+		sprintf ( CharText, " ");
+
+
+
+	    
+
+	    float tmp = calculate_program_effect_duration(SkillOfThisSlot);
+	    if ( tmp > 0 )
+ 	     sprintf(CharText + strlen(CharText), _(" for %c%.1f%c seconds"),  font_switchto_msgvar[0] , tmp , font_switchto_msgstat[0] );
+	   
+	    DisplayText( CharText , 16 + 64 + 16 + SkillScreenRect.x , nextypos , &SkillScreenRect , TEXT_STRETCH );
+	    nextypos += FontHeight( GetCurrentFont() );
+	    }
+
     }
 
     //--------------------
@@ -726,7 +895,7 @@ ShowSkillsScreen ( void )
                               NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible  < number_of_skills )
 		if ( SkillSubsetMap [ CursorIsOnWhichSkillButton ( CurPos.x , CurPos.y ) + 
 			      NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible ] >= 0 ) 
-		    Me[0].readied_skill = SkillSubsetMap [ CursorIsOnWhichSkillButton ( CurPos.x , CurPos.y ) + 
+		    Me.readied_skill = SkillSubsetMap [ CursorIsOnWhichSkillButton ( CurPos.x , CurPos.y ) + 
 						   NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible ] ;
     }
     
@@ -734,6 +903,7 @@ ShowSkillsScreen ( void )
 	 MouseLeftClicked() )
     {
 	toggle_game_config_screen_visibility(GAME_CONFIG_SCREEN_VISIBLE_SKILL_EXPLANATION);
+	while ( MouseLeftPressed () ) SDL_Delay(1);
     }
     
     //--------------------
