@@ -1,8 +1,9 @@
 /*
  *
  *   Copyright (c) 2006 Arvid Picciani
- *   Copyright (c) 1994, 2002, 2003 Johannes Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *   Copyright (c) 1994, 2002 Reinhard Prix
+ *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *
  *
  *  This file is part of Freedroid
@@ -24,9 +25,9 @@
  *
  */
 
-/* ----------------------------------------------------------------------
+/**
  * This file contains sound related functions
- * ---------------------------------------------------------------------- */
+ */
 
 #ifndef _sound_c
 #define _sound_c
@@ -58,7 +59,7 @@ void SwitchBackgroundMusicTo ( char* filename_raw_parameter ){}
 void play_sample_using_WAV_cache( char* SoundSampleFileName , int With_Waiting , int no_double_catching ){}
 void play_sample_using_WAV_cache_v( char* SoundSampleFileName , int With_Waiting , int no_double_catching, double volume ){}
 void remove_all_samples_from_WAV_cache( void ){}
-void PlayOnceNeededSoundSample( char* SoundSampleFileName , int With_Waiting , int no_double_catching ){}
+void PlayOnceNeededSoundSample( const char* SoundSampleFileName , const int With_Waiting , const int no_double_catching ){}
 #endif
 
 
@@ -132,7 +133,7 @@ InitAudio( void )
 		fprintf( stderr, "\n\nSDL just reported a problem.\n\
 				 The error string from SDL_GetError\nwas: %s \n\
 				 The error string from the SDL mixer subsystem was: %s \n" , SDL_GetError() , Mix_GetError() );
-		GiveStandardErrorMessage ( __FUNCTION__ , "\
+		ErrorMessage ( __FUNCTION__ , "\
 								   The SDL AUDIO SUBSYSTEM COULD NOT BE INITIALIZED.\n\
 								   \n\
 								   Please check that your sound card is properly configured,\n\
@@ -161,7 +162,7 @@ InitAudio( void )
 		fprintf( stderr, "\n\nSDL just reported a problem.\n\
 				 The error string from SDL_GetError\nwas: %s \n\
 				 The error string from the SDL mixer subsystem was: %s \n" , SDL_GetError() , Mix_GetError() );
-		GiveStandardErrorMessage ( __FUNCTION__ , "\
+		ErrorMessage ( __FUNCTION__ , "\
 								   The SDL AUDIO CHANNEL COULD NOT BE OPEND.\n\
 								   \n\
 								   Please check that your sound card is properly configured,\n\
@@ -249,10 +250,10 @@ void channelDone( int channel )
 
 
 
-/* ----------------------------------------------------------------------
+/**
  * This function will be invoked by the callback of the SDL mixer when
  * the fading out effect with the background music is completed.
- * ---------------------------------------------------------------------- */
+ */
 void OldMusicHasFinishedFadingOut ( void )
 	{
 
@@ -269,7 +270,7 @@ void OldMusicHasFinishedFadingOut ( void )
 
 
 /** ============================================ MUSIC ============================================ */
-/* ----------------------------------------------------------------------
+/**
  * This function is intended to provide a convenient way of switching
  * between different backround sounds in freedroid.
  * If no background sound was yet running, the function should start playing
@@ -277,7 +278,7 @@ void OldMusicHasFinishedFadingOut ( void )
  * If some background sound was already playing, the function should shut down
  * the old background music and start playing the new one.
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 LoadAndFadeInBackgroundMusic ( void )
 	{
@@ -401,6 +402,37 @@ void SwitchBackgroundMusicTo ( char* filename_raw_parameter )
 
 
 
+// waits for the given amount of time (in milliseconds)
+// can be interrupted by pressing ESC or space or by clicking the left mouse button
+// (also handles window closing events while waiting)
+void WaitInterruptible(const int timeout) {
+    SDL_Event event;
+    int start;
+    
+    if (EscapePressed() || SpacePressed() || MouseLeftPressed()) {
+        return;
+    }
+    start = SDL_GetTicks();
+    while (SDL_GetTicks() - start < timeout ) {
+        while (SDL_PollEvent(&event)) {
+            switch (event.type) {
+                case SDL_QUIT:
+                    Terminate(0);
+                case SDL_KEYDOWN:
+                    if (event.key.keysym.sym == SDLK_ESCAPE ||
+                        event.key.keysym.sym == SDLK_SPACE) {
+                        return;
+                    }
+                    break;
+                case SDL_MOUSEBUTTONDOWN:
+                    if (event.button.button == SDL_BUTTON_LEFT) {
+                        return;
+                    }
+            }
+        }
+        SDL_Delay(10);
+    }    
+}
 
 
 
@@ -412,303 +444,255 @@ void SwitchBackgroundMusicTo ( char* filename_raw_parameter )
 // therefore be loaded and dumped on demand while the other sound samples
 // for the action parts of the game will be kept in memory all the time.
 // ----------------------------------------------------------------------
-void PlayOnceNeededSoundSample( char* SoundSampleFileName , int With_Waiting , int no_double_catching )
-	{
-	int simulated_playback_starting_time;
-	static char PreviousFileName [ 1000 ] = "HalloHallo";
-	static Uint32 PreviousStartTicks = 0 ;
-	Uint32 TicksNow;
-	// This code searches for different kinds of
-	int i;
-	int pathlen;
-	char *extension;
-	char *extensions[] = { ".spx", ".ogg", ".wav", NULL };  // Extensions to try for audio
+void PlayOnceNeededSoundSample( const char* SoundSampleFileName , const int With_Waiting , const int no_double_catching )
+{
+    static char PreviousFileName [ 1000 ] = "HalloHallo";
+    static Uint32 PreviousStartTicks = 0 ;
+    Uint32 TicksNow;
+    // This code searches for different kinds of
+    int i;
+    int pathlen;
+    char *extension;
+    char *extensions[] = { ".spx", ".ogg", ".wav", NULL };  // Extensions to try for audio
 
-	//--------------------
-	// These variables will only be needed when compiling with sound!
-	//
+    //--------------------
+    // These variables will only be needed when compiling with sound!
+    //
 
-	int Newest_Sound_Channel = 0;
-	Mix_Chunk *One_Shot_WAV_File;
-	char Temp_Filename[ 5000 ];
-	char fpath[2048] = "no_fpath_has_been_set";
-
-	//--------------------
-	// In case the same sample is played again and again in a very
-	// short time, we might refuse operation here, since this could
-	// lead to non-loadability errors with the sound files.
-	//
-	TicksNow = SDL_GetTicks();
-	if ( ( strcmp ( "Sorry_No_Voice_Sample_Yet_0.wav" , SoundSampleFileName ) ) && ( ! strcmp ( PreviousFileName , SoundSampleFileName ) ) && ( ( TicksNow - PreviousStartTicks ) < 2.5 * 1000 ) && ( ! no_double_catching ) )
-		return ;
-
-	//--------------------
-	// For now, we disable the bombardment with 'no voice sample yet...'
-	//
-	// if ( ! strcmp ( "Sorry_No_Voice_Sample_Yet_0.wav" , SoundSampleFileName ) ) return;
-
-	PreviousStartTicks = TicksNow;
-	strcpy ( PreviousFileName , SoundSampleFileName );
-
-	//--------------------
-	// In case there are no sound capabilities on this machine, we
-	// wait if that is appropriate or otherwise terminate immediately.
-	//
-///aep: no idea whats the point of it
-/* 
-#ifndef HAVE_LIBSDL_MIXER
-
-	if ( With_Waiting )
-		{
-		simulated_playback_starting_time = SDL_GetTicks() ;
-
-		while ( ( SDL_GetTicks() - simulated_playback_starting_time < 7 * 1000 ) &&
-		        !EscapePressed() && !SpacePressed() )
-			;
-
-		if ( EscapePressed() )
-			while ( EscapePressed() )
-				;
-		if ( SpacePressed() )
-			while ( SpacePressed() )
-				;
-		}
+    int Newest_Sound_Channel = 0;
+    Mix_Chunk *One_Shot_WAV_File;
+    char Temp_Filename[ 5000 ];
+    char fpath[2048] = "no_fpath_has_been_set";
 
+    //--------------------
+    // In case the same sample is played again and again in a very
+    // short time, we might refuse operation here, since this could
+    // lead to non-loadability errors with the sound files.
+    //
+    TicksNow = SDL_GetTicks();
+    if ( ( strcmp ( "Sorry_No_Voice_Sample_Yet_0.wav" , SoundSampleFileName ) ) && ( ! strcmp ( PreviousFileName , SoundSampleFileName ) ) && ( ( TicksNow - PreviousStartTicks ) < 2.5 * 1000 ) && ( ! no_double_catching ) )
 	return ;
-#else
-*/
-	if ( !sound_on )
+
+    //--------------------
+    // For now, we disable the bombardment with 'no voice sample yet...'
+    //
+    // if ( ! strcmp ( "Sorry_No_Voice_Sample_Yet_0.wav" , SoundSampleFileName ) ) return;
+
+    PreviousStartTicks = TicksNow;
+    strcpy ( PreviousFileName , SoundSampleFileName );
+
+    if ( !sound_on )
+	{
+	//--------------------
+	// Maybe this sound sample was intended to be hooking the CPU and the
+	// program flow, so that nothing happens until the sample has been
+	// played fully.  In this case we must introduce a waiting time even
+	// now that no sound sample is played.  A default of 7 seconds seems to
+	// be appropriate.  On pressing the left button or space or escape
+	// the waiting time will be cancelled anyway.
+	//
+	if ( With_Waiting ) {
+            WaitInterruptible(7000);
+        }
+
+	//--------------------
+	// Since sound is disabled otherwise we MUST return here and not
+	// try to do any sound operations on this machine with perhaps no sound
+	// modules and no SDL sound initialized.
+	//
+	return ;
+	}
+
+    //--------------------
+    // Now we set a callback function, that should be called by SDL
+    // as soon as ANY other sound channel finishes playing...
+    //
+    Mix_ChannelFinished( channelDone );
+
+    //--------------------
+    // Now we try to load the requested sound file into memory...
+    //
+    One_Shot_WAV_File = NULL;
+
+    strcpy ( Temp_Filename , SoundSampleFileName );
+
+    //--------------------
+    // Only if the file name wasn't 'no_voice_sample', we really
+    // try to load anything...
+    //
+    if ( strcmp ( SoundSampleFileName , "Sorry_No_Voice_Sample_Yet_0.wav" ) )
+	{
+	pathlen = strlen ( Temp_Filename );
+
+	if ( strcmp ( Temp_Filename + pathlen - 4 , ".ogg" ) == 0 )
+	    {
+	    extension = Temp_Filename + pathlen - 4;
+	    }
+	else
+	    {
+	    extension = Temp_Filename + pathlen;
+	    }
+
+	i = 0;
+	while ( extensions [ i ] != NULL )
+	    {
+	    strcpy ( extension, extensions [ i ] );
+	    //--------------------
+	    // find_file_silent may return a NULL pointer, in case the file name
+	    // composed hasn't been found.  We need to catch that case of course.
+	    //
+	    if ( find_file ( Temp_Filename , SOUND_DIR, fpath, 1 ) == 0)
 		{
-		//--------------------
-		// Maybe this sound sample was intended to be hooking the CPU and the
-		// program flow, so that nothing happens until the sample has been
-		// played fully.  In this case we must introduce a waiting time even
-		// now that no sound sample is played.  A default of 7 seconds seems to
-		// be appropriate.  On pressing the left button or space or escape
-		// the waiting time will be cancelled anyway.
-		//
-		if ( With_Waiting )
-			{
-			simulated_playback_starting_time = SDL_GetTicks() ;
-
-			while ( ( SDL_GetTicks() - simulated_playback_starting_time < 7 * 1000 ) &&
-					!EscapePressed() && !SpacePressed() && !MouseLeftPressed())
-				SDL_Delay( 10 );
-
-			while ( EscapePressed() || SpacePressed() || MouseLeftPressed() )
-					SDL_Delay(1);
-			}
-
-		//--------------------
-		// Since sound is disabled otherwise we MUST return here and not
-		// try to do any sound operations on this machine with perhaps no sound
-		// modules and no SDL sound initialized.
-		//
-		return ;
+		One_Shot_WAV_File = Mix_LoadWAV ( fpath ) ;
+		if ( One_Shot_WAV_File != NULL )
+		    {
+		    break;
+		    }
+		else
+		    {
+		    ErrorMessage ( __FUNCTION__ , "\
+			    Corrupt sound file encountered!  The file is there, \n\
+			    but the SDL MIXER was unable to LOAD it.",
+			    NO_NEED_TO_INFORM , IS_WARNING_ONLY );
+		    }
 		}
 
-	//--------------------
-	// Now we set a callback function, that should be called by SDL
-	// as soon as ANY other sound channel finishes playing...
-	//
-	Mix_ChannelFinished( channelDone );
+	    i++;
+	    }
+	}
+    else
+	One_Shot_WAV_File = NULL ;
 
+    //--------------------
+    // Now some error checking against failed/missing sound samples...
+    //
+    if ( One_Shot_WAV_File == NULL )
+	{
 	//--------------------
-	// Now we try to load the requested sound file into memory...
-	//
-	One_Shot_WAV_File = NULL;
-
-	strcpy ( Temp_Filename , SoundSampleFileName );
-
-	//--------------------
-	// Only if the file name wasn't 'no_voice_sample', we really
-	// try to load anything...
+	// A warning message about a missing speech file should only be issued,
+	// if it wasn't the 'no_voice_sample' dummy entry anyway...
 	//
 	if ( strcmp ( SoundSampleFileName , "Sorry_No_Voice_Sample_Yet_0.wav" ) )
-		{
-		pathlen = strlen ( Temp_Filename );
-
-		if ( strcmp ( Temp_Filename + pathlen - 4 , ".ogg" ) == 0 )
-			{
-			extension = Temp_Filename + pathlen - 4;
-			}
-		else
-			{
-			extension = Temp_Filename + pathlen;
-			}
-
-		i = 0;
-		while ( extensions [ i ] != NULL )
-			{
-			strcpy ( extension, extensions [ i ] );
-			//--------------------
-			// find_file_silent may return a NULL pointer, in case the file name
-			// composed hasn't been found.  We need to catch that case of course.
-			//
-                       if ( find_file ( Temp_Filename , SOUND_DIR, fpath, 1 ) == 0)
-				{
-				One_Shot_WAV_File = Mix_LoadWAV ( fpath ) ;
-				if ( One_Shot_WAV_File != NULL )
-					{
-					break;
-					}
-				else
-					{
-					GiveStandardErrorMessage ( __FUNCTION__ , "\
-Corrupt sound file encountered!  The file is there, \n\
-but the SDL MIXER was unable to LOAD it.",
-											   NO_NEED_TO_INFORM , IS_WARNING_ONLY );
-					}
-				}
-
-			i++;
-			}
-		}
-	else
-		One_Shot_WAV_File = NULL ;
-
+	    {
+	    fprintf( stderr, "\n\nSoundSampleFileName: '%s'" , SoundSampleFileName );
+	    ErrorMessage ( __FUNCTION__ , "\
+		    There seems to be a sound file missing.",
+		    NO_NEED_TO_INFORM, FALSE );
+	    }
 	//--------------------
-	// Now some error checking against failed/missing sound samples...
-	//
-	if ( One_Shot_WAV_File == NULL )
-		{
-		//--------------------
-		// A warning message about a missing speech file should only be issued,
-		// if it wasn't the 'no_voice_sample' dummy entry anyway...
-		//
-		if ( strcmp ( SoundSampleFileName , "Sorry_No_Voice_Sample_Yet_0.wav" ) )
-			{
-			fprintf( stderr, "\n\nSoundSampleFileName: '%s'" , SoundSampleFileName );
-                       GiveStandardErrorMessage ( __FUNCTION__ , "\
-There seems to be a sound file missing.",
-									   NO_NEED_TO_INFORM, GameConfig.terminate_on_missing_speech_sample );
-			}
-
-		if ( !GameConfig.terminate_on_missing_speech_sample )
-			{
-			//--------------------
-			// Maybe this sound sample was intended to be hooking the CPU and the
-			// program flow, so that nothing happens until the sample has been
-			// played fully.  In this case we must introduce a waiting time even
-			// if no sound sample is played.  A default of 7 seconds seems to
-			// be appropriate.  On pressing the left button or space or escape
-			// the waiting time will be cancelled anyway.
-			//
-			if ( With_Waiting )
-				{
-				simulated_playback_starting_time = SDL_GetTicks() ;
-
-				while ( ( SDL_GetTicks() - simulated_playback_starting_time < 7 * 1000 ) &&
-						!EscapePressed() && !SpacePressed() && !MouseLeftPressed() )
-					SDL_Delay(10);
-
-				while ( EscapePressed() || SpacePressed() || MouseLeftPressed() )
-					SDL_Delay(10);
-				}
-
-			//--------------------
-			// Now we must return, since we do not want to 'free' the sound sample, that
-			// hasn't been loaded successfully and produce a segfault, do we?
-			//
-			return ;
-
-			}
-		} // if ( !Loaded_WAV...
-	else
-		{
-		DebugPrintf ( 1 , "\nSuccessfully loaded file %s into memory for playing once, filename is %s ." ,
-					  SoundSampleFileName , fpath );
-		}
-
-	//--------------------
-	// Hoping, that this will not take up too much processor speed, we'll
-	// now change the volume of the sound sample in question to what is normal
-	// for sound effects right now...
-	//
-	// And of course we may only do this, if the sound file has been loaded
-	// successfully!  Otherwise the SDL_mixer lib will produce a segfault.
-	//
-	if ( One_Shot_WAV_File != NULL )
-		Mix_VolumeChunk( One_Shot_WAV_File ,
-						 ( int ) rintf( GameConfig.Current_Sound_FX_Volume * MIX_MAX_VOLUME ) );
-
-	//--------------------
-	// Now we try to play the sound file that has just been successfully
-	// loaded into memory...
-	//
-	// In case of an error, we will of course print an error message
-	// and quit...
-	//
-	Newest_Sound_Channel = Mix_PlayChannel( -1 , One_Shot_WAV_File , 0 );
-	if ( Newest_Sound_Channel <= -1 )
-		{
-		fprintf( stderr, "\n\nSoundSampleFileName: '%s' Mix_GetError(): %s \n" , SoundSampleFileName , Mix_GetError() );
-		GiveStandardErrorMessage ( __FUNCTION__ , "\
-								   The SDL MIXER WAS UNABLE TO PLAY A CERTAIN FILE LOADED INTO MEMORY FOR PLAYING ONCE.\n",
-								   PLEASE_INFORM, IS_WARNING_ONLY );
-
-		//--------------------
-		// If we receive an error playing a sound file here, this is very inconvenient.
-		// We must see to it that the callback code and allocation there and all that doesn't
-		// get touched.  I hope that the following fix does already what we want here...
-		// But it should :->
-		//
-		Mix_FreeChunk ( One_Shot_WAV_File );
-		return ;
-
-		} // if ( ... = -1
-	else
-		{
-		SoundChannelList[ Newest_Sound_Channel ] = 1;
-		DebugPrintf( 1 , "\nSuccessfully playing the 'ONCE NEEDED' file %s.", SoundSampleFileName ) ;
-		}
-
-	//--------------------
-	// Maybe this sound sample is intended to be hooking the CPU and the
+	// Maybe this sound sample was intended to be hooking the CPU and the
 	// program flow, so that nothing happens until the sample has been
-	// played fully...
+	// played fully.  In this case we must introduce a waiting time even
+	// if no sound sample is played.  A default of 7 seconds seems to
+	// be appropriate.  On pressing the left button or space or escape
+	// the waiting time will be cancelled anyway.
 	//
-	if ( With_Waiting )
-		{
-		while ( SoundChannelList[ Newest_Sound_Channel ] && !EscapePressed() && !SpacePressed() )
-			;
-		//--------------------
-		// In case escape was pressed, the currently playing voice sample must
-		// be terminated immediately.
-		//
-		if ( EscapePressed() || SpacePressed() || MouseLeftPressed() )
-			{
-			Mix_HaltChannel( Newest_Sound_Channel );
-			while ( EscapePressed() || SpacePressed() || MouseLeftPressed())
-				SDL_Delay(10);
+	if ( With_Waiting ) {
+            WaitInterruptible(7000);
+        }
 
-			//--------------------
-			// Now the channel has finished playing (or we have stopped it) and
-			// now we can unallocate the resources used by it...
-			//
-			if ( One_Shot_WAV_File != NULL )
-				Mix_FreeChunk ( One_Shot_WAV_File );
+	//--------------------
+	// Now we must return, since we do not want to 'free' the sound sample, that
+	// hasn't been loaded successfully and produce a segfault, do we?
+	//
+	return ;
 
-			}
-		}
-	else
-		{
-		//--------------------
-		// Otherwise, if there was no 'With_Waiting' flag set,
-		// we do nothing here, cause we can't halt the channel and
-		// we also can't free the channel, that is still playing.
-		//
-		// All we will do is set the channels flag to 2, so that the
-		// callback function will know what to do when called:  TO
-		// FREE THE SOUND CHUNK!
-		//
-		SoundChannelList[ Newest_Sound_Channel ] = 2;
-		List_Of_Sustained_Release_WAV_Files[ Newest_Sound_Channel ] = One_Shot_WAV_File ;
-		}
+	} // if ( !Loaded_WAV...
+    else
+	{
+	DebugPrintf ( 1 , "\nSuccessfully loaded file %s into memory for playing once, filename is %s ." ,
+		SoundSampleFileName , fpath );
+	}
+
+    //--------------------
+    // Hoping, that this will not take up too much processor speed, we'll
+    // now change the volume of the sound sample in question to what is normal
+    // for sound effects right now...
+    //
+    // And of course we may only do this, if the sound file has been loaded
+    // successfully!  Otherwise the SDL_mixer lib will produce a segfault.
+    //
+    if ( One_Shot_WAV_File != NULL )
+	Mix_VolumeChunk( One_Shot_WAV_File ,
+		( int ) rintf( GameConfig.Current_Sound_FX_Volume * MIX_MAX_VOLUME ) );
+
+    //--------------------
+    // Now we try to play the sound file that has just been successfully
+    // loaded into memory...
+    //
+    // In case of an error, we will of course print an error message
+    // and quit...
+    //
+    Newest_Sound_Channel = Mix_PlayChannel( -1 , One_Shot_WAV_File , 0 );
+    if ( Newest_Sound_Channel <= -1 )
+	{
+	fprintf( stderr, "\n\nSoundSampleFileName: '%s' Mix_GetError(): %s \n" , SoundSampleFileName , Mix_GetError() );
+	ErrorMessage ( __FUNCTION__ , "\
+		The SDL MIXER WAS UNABLE TO PLAY A CERTAIN FILE LOADED INTO MEMORY FOR PLAYING ONCE.\n",
+		PLEASE_INFORM, IS_WARNING_ONLY );
+
+	//--------------------
+	// If we receive an error playing a sound file here, this is very inconvenient.
+	// We must see to it that the callback code and allocation there and all that doesn't
+	// get touched.  I hope that the following fix does already what we want here...
+	// But it should :->
+	//
+	Mix_FreeChunk ( One_Shot_WAV_File );
+	return ;
+
+	} // if ( ... = -1
+    else
+	{
+	SoundChannelList[ Newest_Sound_Channel ] = 1;
+	DebugPrintf( 1 , "\nSuccessfully playing the 'ONCE NEEDED' file %s.", SoundSampleFileName ) ;
+	}
+
+    //--------------------
+    // Maybe this sound sample is intended to be hooking the CPU and the
+    // program flow, so that nothing happens until the sample has been
+    // played fully...
+    //
+    if ( With_Waiting )
+	{
+	while ( SoundChannelList[ Newest_Sound_Channel ] && !EscapePressed() && !SpacePressed() )
+	    ;
+	//--------------------
+	// In case escape was pressed, the currently playing voice sample must
+	// be terminated immediately.
+	//
+	if ( EscapePressed() || SpacePressed() || MouseLeftPressed() )
+	    {
+	    Mix_HaltChannel( Newest_Sound_Channel );
+	    while ( EscapePressed() || SpacePressed() || MouseLeftPressed())
+		SDL_Delay(10);
+
+	    //--------------------
+	    // Now the channel has finished playing (or we have stopped it) and
+	    // now we can unallocate the resources used by it...
+	    //
+	    if ( One_Shot_WAV_File != NULL )
+		Mix_FreeChunk ( One_Shot_WAV_File );
+
+	    }
+	}
+    else
+	{
+	//--------------------
+	// Otherwise, if there was no 'With_Waiting' flag set,
+	// we do nothing here, cause we can't halt the channel and
+	// we also can't free the channel, that is still playing.
+	//
+	// All we will do is set the channels flag to 2, so that the
+	// callback function will know what to do when called:  TO
+	// FREE THE SOUND CHUNK!
+	//
+	SoundChannelList[ Newest_Sound_Channel ] = 2;
+	List_Of_Sustained_Release_WAV_Files[ Newest_Sound_Channel ] = One_Shot_WAV_File ;
+	}
 
 
 
-	}; // void PlayOnceNeededSoundSample( char* SoundSampleFileName , int With_Waiting)
+}; // void PlayOnceNeededSoundSample( char* SoundSampleFileName , int With_Waiting)
 
 
 
@@ -792,7 +776,7 @@ play_sample_using_WAV_cache_v( char* SoundSampleFileName , int With_Waiting , in
 		if ( dynamic_WAV_cache [ next_free_position_in_cache ] == NULL )
 			{
 			fprintf( stderr, "\n\nfpath: '%s'\n" , fpath );
-			GiveStandardErrorMessage ( __FUNCTION__ , "\
+			ErrorMessage ( __FUNCTION__ , "\
 			                           The SDL MIXER WAS UNABLE TO LOAD A CERTAIN SOUND FILE INTO MEMORY.\n\
 			                           This should not happen for samples that are supposed to be cached...",
 			                           NO_NEED_TO_INFORM , IS_WARNING_ONLY );
@@ -831,7 +815,7 @@ play_sample_using_WAV_cache_v( char* SoundSampleFileName , int With_Waiting , in
 		if ( next_free_position_in_cache >= MAX_SOUNDS_IN_DYNAMIC_WAV_CACHE )
 			{
 			fprintf( stderr, "\n\nnext_free_position_in_cache: %d,\n" , next_free_position_in_cache );
-			GiveStandardErrorMessage ( __FUNCTION__ , "\
+			ErrorMessage ( __FUNCTION__ , "\
 			                           ALERT!  Ran out of space in the dynamic wav sample cache!  Cache size too small?",
 			                           PLEASE_INFORM, IS_FATAL );
 			}
@@ -848,7 +832,7 @@ play_sample_using_WAV_cache_v( char* SoundSampleFileName , int With_Waiting , in
 	if ( Newest_Sound_Channel <= -1 )
 		{
 		fprintf( stderr, "\n\nSoundSampleFileName: '%s' Mix_GetError(): %s \n" , SoundSampleFileName , Mix_GetError() );
-		GiveStandardErrorMessage ( __FUNCTION__ , "\
+		ErrorMessage ( __FUNCTION__ , "\
 		                           The SDL mixer was unable to play a certain sound sample file, that was supposed to be cached for later.\n",
 		                           NO_NEED_TO_INFORM, IS_WARNING_ONLY );
 		} // if ( ... = -1
@@ -862,7 +846,7 @@ play_sample_using_WAV_cache_v( char* SoundSampleFileName , int With_Waiting , in
 ; // void play_sample_using_WAV_cache(...)
 
 
-/* ----------------------------------------------------------------------
+/**
  * When the sound sample volume is changed via the in-game controls, we 
  * need to re-sample everything, which is done only upon loading of the
  * sound samples.  Instead of RE-MIXING SOUND SAMPLES AGAIN AND AGAIN to
@@ -870,7 +854,7 @@ play_sample_using_WAV_cache_v( char* SoundSampleFileName , int With_Waiting , in
  * out the current wav file cache, so that when needed again, the sound
  * files will be loaded again and then automatically re-sampled to proper
  * volume from the play/load function itself.  Perfect solution.
- * ---------------------------------------------------------------------- */
+ */
 void
 remove_all_samples_from_WAV_cache( void )
 	{
