@@ -1,6 +1,7 @@
 /* 
  *
  *   Copyright (c) 2002, 2003  Reinhard Prix
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -48,6 +49,10 @@
 #include <string.h>
 #endif
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
 #ifdef HAVE_TIME_H
 #include <time.h>
 #else 
@@ -87,10 +92,20 @@
 //#include <locale.h>
 //#endif
 
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
+
+#ifdef HAVE_LIBGEN_H
+#include <libgen.h>
+#endif
+
 #include <errno.h>
 #include <stdarg.h>
 #include <ctype.h>
 
+#include <setjmp.h>
+
 #ifdef HAVE_DIRENT_H
 #include "dirent.h"
 #endif
@@ -102,10 +117,6 @@
 #include "SDL_mixer.h"
 #endif
 
-#ifdef HAVE_LIBSDL_NET
-#include "SDL_net.h"
-#endif
-
 #ifdef HAVE_LIBGL
 #include "SDL_opengl.h"
 #endif /* HAVE_LIBGL */
