@@ -1,5 +1,6 @@
 /* 
  *
+ *   Copyright (c) 2004-2007 Arthur Huillet 
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
  *
@@ -22,10 +23,10 @@
  *  MA  02111-1307  USA
  *
  */
-/* ----------------------------------------------------------------------
+/**
  * This file does everything that has to do with the takeover game from
  * the original paradroid game.
- * ---------------------------------------------------------------------- */
+ */
 
 #define _takeover_c
 
@@ -36,9 +37,9 @@
 #include "global.h"
 #include "proto.h"
 #include "takeover.h"
-#include "ship.h"
 #include "map.h"
 
+enemy * cDroid; 
 Uint32 cur_time;  		// current time in ms 
 SDL_Surface *to_blocks;         // the global surface containing all game-blocks 
 
@@ -121,7 +122,7 @@ int LeaderColor = GELB;		/* momentary leading color */
 int YourColor = GELB;
 int OpponentColor = VIOLETT;
 int OpponentType;		/* The druid-type of your opponent */
-int DroidNum;
+enemy * cdroid;
 
 /* the display  column */
 int DisplayColumn[NUM_LINES] = {
@@ -149,24 +150,26 @@ void AdvancedEnemyTakeoverMovements (void);
  *
  *-----------------------------------------------------------------*/
 int
-Takeover ( int enemynum )
+Takeover ( enemy * target )
 {
     int row;
     int FinishTakeover = FALSE;
     char *message;
-    int ClearanceIndex;
     int Finished = FALSE;
-    int WasPressed = FALSE ;
     int Displacement = 0 ;
     int reward = 0;
-    char game_message_text [ 5000 ] ;
+    char game_message_text [ 500 ] ;
+    SDL_Event event;
+    int old_status;
+
+    old_status = game_status;
 
     //--------------------
     // Prevent distortion of framerate by the delay coming from 
     // the time spent in the menu.
     //
     Activate_Conservative_Frame_Computation ();
-    
+
     //--------------------
     // We set the UserRect to full again, no matter what other windows might
     // be open right now...
@@ -175,88 +178,101 @@ Takeover ( int enemynum )
     User_Rect . y = 0 ;
     User_Rect . w = GameConfig . screen_width ;
     User_Rect . h = GameConfig . screen_height ;
-    
+
     //--------------------
     // Maybe takeover graphics haven't been loaded yet.  Then we do this
     // here now and for once.  Later calls will be ignored inside the function.
     //
     GetTakeoverGraphics ( ) ;
-    
-    while (SpacePressed ()) ;  // make sure space is release before proceed 
-    
+
+    while (SpacePressed () || MouseLeftPressed()) ;  // make sure space is release before proceed 
+
     SwitchBackgroundMusicTo ( TAKEOVER_BACKGROUND_MUSIC_SOUND ); // now this is a STRING!!!
-    
+
     DisplayBanner ( ) ;
-    Me [ 0 ] . status = MOBILE; // the new status _after_ the takeover game 
-    
+    Me . status = MOBILE; // the new status _after_ the takeover game 
+
     //--------------------
     // Now it is time to display the enemy of this whole takeover process...
     //
-    while ( !Finished )
-    {
-	ShowDroidInfo ( AllEnemys[enemynum].type, Displacement , TRUE );
+    if ( GameConfig . auto_display_to_help ) 
+	{
+	PlayATitleFile ( "TakeoverInstructions.title" );
+	GameConfig . auto_display_to_help = 0;
+	}
+
+    while ( !Finished )	{
+	ShowDroidInfo ( target->type, Displacement , TRUE );
 	ShowGenericButtonFromList ( TAKEOVER_HELP_BUTTON ) ;
 	blit_our_own_mouse_cursor ( );
-	our_SDL_flip_wrapper ( Screen );
-	
-	if ( MouseCursorIsOnButton( UP_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && axis_is_active && !WasPressed )
-	{
-	    MoveMenuPositionSound();
-	    Displacement += FontHeight ( GetCurrentFont () );
+	our_SDL_flip_wrapper();
+
+	while(SDL_PollEvent(&event)) {
+
+	    if (event.type == SDL_QUIT) {
+		Terminate(0);
+	    }
+
+	    if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == SDL_BUTTON_LEFT) {
+		if ( MouseCursorIsOnButton( UP_BUTTON , GetMousePos_x()  , GetMousePos_y()  ))
+		    {
+		    MoveMenuPositionSound();
+		    Displacement += FontHeight ( GetCurrentFont () );
+		    }
+		else if ( MouseCursorIsOnButton( DOWN_BUTTON , GetMousePos_x()  , GetMousePos_y()  ))
+		    {
+		    MoveMenuPositionSound();
+		    Displacement -= FontHeight ( GetCurrentFont () );
+		    }
+		else if ( MouseCursorIsOnButton( DRUID_SHOW_EXIT_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  )) 
+		    {
+		    Finished = TRUE;
+		    }
+		else if ( MouseCursorIsOnButton( TAKEOVER_HELP_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  )) 
+		    {
+		    PlayATitleFile ( "TakeoverInstructions.title" );
+		    }
+	    } else if (event.type == SDL_KEYDOWN && ((event.key.keysym.sym == SDLK_SPACE) || (event.key.keysym.sym == SDLK_ESCAPE))) {
+		Finished = TRUE ;
+	    }	
 	}
-	else if ( MouseCursorIsOnButton( DOWN_BUTTON , GetMousePos_x()  , GetMousePos_y()  ) && axis_is_active && !WasPressed )
-	{
-	    MoveMenuPositionSound();
-	    Displacement -= FontHeight ( GetCurrentFont () );
-	}
-	else if ( MouseCursorIsOnButton( DRUID_SHOW_EXIT_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  ) && axis_is_active && !WasPressed ) 
-	{
-	    Finished = TRUE;
-	}
-	else if ( MouseCursorIsOnButton( TAKEOVER_HELP_BUTTON , GetMousePos_x ( )  , GetMousePos_y ( )  ) && axis_is_active && !WasPressed ) 
-	{
-	    PlayATitleFile ( "TakeoverInstructions.title" );
-	}
-	WasPressed = axis_is_active; 
-	
-	if ( SpacePressed() && !axis_is_active ) Finished = TRUE ;
-	
+	SDL_Delay(1);
     }
-    
-    while ( !( !SpacePressed() && !EscapePressed() && !axis_is_active )) ;
-    
-    
+
+    while ( !( !SpacePressed() && !EscapePressed() && !MouseLeftPressed() )) ;
+
+
     while (!FinishTakeover)
-    {
+	{
 	//--------------------
 	// Init Color-column and Capsule-Number for each opponenet and your color 
 	//
 	for (row = 0; row < NUM_LINES; row++)
-	{
+	    {
 	    DisplayColumn[row] = (row % 2);
 	    CapsuleCountdown[GELB][0][row] = -1;
 	    CapsuleCountdown[VIOLETT][0][row] = -1;
-	} // for row 
-	
+	    } // for row 
+
 	YourColor = GELB;
 	OpponentColor = VIOLETT;
-	
+
 	CapsuleCurRow[GELB] = 0;
 	CapsuleCurRow[VIOLETT] = 0;
-	
-	DroidNum = enemynum;
-	OpponentType = AllEnemys [ enemynum ] . type;
-	// NumCapsules[YOU] = 3 + ClassOfDruid (Me[0].type);
-	NumCapsules[YOU] = 3 + Me [ 0 ] . hacking_skill ;
+
+	OpponentType = target->type;
+	cDroid = target;
+	// NumCapsules[YOU] = 3 + ClassOfDruid (Me.type);
+	NumCapsules[YOU] = 2 + Me . base_skill_level [ get_program_index_with_name("Hacking") ];
 	NumCapsules[ENEMY] = 2 + Druidmap [ OpponentType ] . class ;
-	
+
 	InventPlayground ();
-	
+
 	EvaluatePlayground ();
-	
+
 	ShowPlayground ();
-	our_SDL_flip_wrapper (Screen);
-	
+	our_SDL_flip_wrapper();
+
 	ChooseColor ();
 
 	//--------------------
@@ -267,55 +283,37 @@ Takeover ( int enemynum )
 	// the game must be played again in the next loop...
 	//
 	PlayGame ();
-	
+
 	//--------------------
 	// We we evaluate the final score of the game.  Maybe we're done
 	// already, maybe not...
 	//
 	if ( LeaderColor == YourColor ) 
-	{
+	    {
 	    // SwitchBackgroundMusicTo (SILENCE);
 	    Takeover_Game_Won_Sound ();
-	    
+
 	    //--------------------
 	    // We allow to gain the current energy/full health that was still in the 
 	    // other droid, since all previous damage must be due to fighting damage,
 	    // and this is exactly the sort of damage can usually be cured in refreshes.
 	    //
-	    Me [ 0 ] . energy += AllEnemys [ enemynum ] . energy;
-	    Me [ 0 ] . health += Druidmap [ OpponentType ] . maxenergy;
-	    
+	    Me . energy += target->energy;
+
 	    //--------------------
 	    // We provide some security agains too high energy/health values gained
 	    // by very rapid successions of successful takeover attempts
 	    //
-	    if ( Me [ 0 ] . energy > Me [ 0 ] . maxenergy ) 
-		Me [ 0 ] . energy = Me [ 0 ] . maxenergy;
-	    if ( Me [ 0 ] . health > Me [ 0 ] . maxenergy ) 
-		Me [ 0 ] . health = Me [ 0 ] . maxenergy;
-	    
-	    //--------------------
-	    // We add the victims security clearance to our own list of 
-	    // availabe security clearances.
-	    //
-	    for ( ClearanceIndex = 0 ; ClearanceIndex < MAX_CLEARANCES ; ClearanceIndex ++ )
-	    {
-		if ( Me [ 0 ] . clearance_list [ ClearanceIndex ] == 0 )
-		{
-		    Me [ 0 ] . clearance_list [ ClearanceIndex ] = AllEnemys [ enemynum ] . type ;
-		    // SetNewBigScreenMessage ( "Clearance obtained" );
-		    SetNewBigScreenMessage ( "Bot Reprogrammed" );
-		    break;
-		}
-	    }
-	    
-	    Me [ 0 ] . type = AllEnemys [ enemynum ] . type;
-	    Me [ 0 ] . marker = AllEnemys [ enemynum ] . marker;
+	    if ( Me . energy > Me . maxenergy ) 
+		Me . energy = Me . maxenergy;
 
-	    reward = Druidmap [ AllEnemys [ enemynum ] . type ] . experience_reward * 3 ;
-	    Me [ 0 ] . Experience += reward;
-	    sprintf ( game_message_text , "For taking control of your enemy, you receive %d experience." ,
-		      reward );
+	    Me . type = target->type;
+	    Me . marker = target->marker;
+
+	    reward = Druidmap [ target->type ] . experience_reward * 1 ;
+	    Me . Experience += reward;
+	    sprintf ( game_message_text , _("For taking control of your enemy, you receive %d experience."),
+		    reward );
 	    append_new_game_message ( game_message_text );
 
 	    //--------------------
@@ -325,35 +323,28 @@ Takeover ( int enemynum )
 	    // should be dropped after the successful takeover process, even if the
 	    // enemy isn't completely dead yet...
 	    //
-	    if ( AllEnemys [ enemynum ] . on_death_drop_item_code != (-1) )
-	    {
-		DropItemAt( AllEnemys [ enemynum ] . on_death_drop_item_code , 
-			    AllEnemys [ enemynum ] . pos . z ,
-			    AllEnemys [ enemynum ] . pos . x , 
-			    AllEnemys [ enemynum ] . pos . y , -1 , -1 , 2 , 1 );
-		AllEnemys [ enemynum ] . on_death_drop_item_code = -1;
-	    }  
+	    if ( target->on_death_drop_item_code != (-1) )
+		{
+		DropItemAt( target->on_death_drop_item_code , 
+			target->pos . z ,
+			target->pos . x , 
+			target->pos . y , -1 , -1 , 1 );
+		target->on_death_drop_item_code = -1;
+		}  
 
-	    AllEnemys [ enemynum ] . energy =  Druidmap [ AllEnemys [ enemynum ] . type ] . maxenergy ; 
+	    target->energy =  Druidmap [ target->type ] . maxenergy ; 
 
-	    //--------------------
-	    // After takeover, the enemy suddenly becomes friendly and will also have
-	    // a much nicer default 'combat state'...  the enemy can still start to
-	    // fight from that nicer state, maybe on the Tux side, this time :)
-	    //
-	    AllEnemys [ enemynum ] . is_friendly = TRUE ;
-	    AllEnemys [ enemynum ] . has_been_taken_over = TRUE ; 
-	    if ( AllEnemys [ enemynum ] .  stick_to_waypoint_system_by_default )
-		AllEnemys [ enemynum ] . combat_state = MOVE_ALONG_RANDOM_WAYPOINTS ;
-	    else
-		AllEnemys [ enemynum ] . combat_state = WAYPOINTLESS_WANDERING ;
+	    target->is_friendly = TRUE ;
+	    target->has_been_taken_over = TRUE ; 
+
+	    target->combat_state = WAYPOINTLESS_WANDERING ;
 
 	    //--------------------
 	    // When the bot is taken over, it should not turn hostile when
 	    // the rest of his former combat group (identified by having the
 	    // same marker) is attacked by the Tux.
 	    //
-	    AllEnemys [ enemynum ] . marker = 0 ;
+	    target->marker = 0 ;
 
 	    //--------------------
 	    // The takeover game earlier had some status banner, where the 
@@ -361,57 +352,61 @@ Takeover ( int enemynum )
 	    // very much relevant any more in the freedroidRPG variant....
 	    //
 	    if ( LeaderColor != YourColor )	// only won because of InvincibleMode 
-		message = "You cheat";
+		message = _("You cheat");
 	    else				// won the proper way 
-		message = "Complete";
-	    
+		message = _("Complete");
+
 	    //--------------------
 	    // Ok.  The whole takeover is done now.  We can return...
 	    //
 	    FinishTakeover = TRUE;
-	}				// LeaderColor == YourColor 
+	    }				// LeaderColor == YourColor 
 	else if ( LeaderColor == OpponentColor )
-	{
+	    {
 	    Takeover_Game_Lost_Sound ();
-	    message = "Rejected";
-	    Me [ 0 ] . energy *= 0.5 ;
-	    AllEnemys [ enemynum ] . energy = Druidmap [ AllEnemys [ enemynum ] . type ] . maxenergy ;
+	    message = _("Rejected");
+	    Me . energy *= 0.5 ;
+	    target->energy = Druidmap [ target->type ] . maxenergy ;
 	    FinishTakeover = TRUE;
-	}			// if LeadColor == OpponentColor 
+	    }			// if LeadColor == OpponentColor 
 	else
-	{
+	    {
 	    Takeover_Game_Deadlock_Sound ();
-	    message = "Deadlock";
-	}			// LeadColor == DRAW	
+	    message = _("Deadlock");
+	    }			// LeadColor == DRAW	
 
 	ShowPlayground ();
 	to_show_banner (message, NULL);
-	our_SDL_flip_wrapper (Screen);
-	
-    }	// while !FinishTakeover 
-    
+	our_SDL_flip_wrapper();
+	SDL_Delay(100);
+
+	}
+
     ClearGraphMem();
-    
-    SwitchBackgroundMusicTo ( CurLevel -> Background_Song_Name );
-    
+
+    SwitchBackgroundMusicTo ( CURLEVEL -> Background_Song_Name );
+
+    game_status = old_status;
+
     if ( LeaderColor == YourColor )
 	return TRUE;
     else
 	return FALSE;
-    
+
 }; // int Takeover( int enemynum ) 
 
 
-/* ----------------------------------------------------------------------
+/**
  * This function does the countdown where you still can changes your
  * color.
- * ---------------------------------------------------------------------- */
+ */
 void
 ChooseColor (void)
 {
     int countdown = 100;  // duration in 1/10 seconds given for color choosing 
     int ColorChosen = FALSE;
     char count_text[80];
+    SDL_Event event;
     
     Uint32 prev_count_tick, count_tick_len;
     
@@ -419,44 +414,74 @@ ChooseColor (void)
 
     prev_count_tick = SDL_GetTicks ();
 
-    while (!ColorChosen)
-    {
+    while (!ColorChosen) {
+
 	// wait for next countdown tick 
-	while ( SDL_GetTicks() < prev_count_tick + count_tick_len ); 
+	while ( SDL_GetTicks() < prev_count_tick + count_tick_len );
 	
 	prev_count_tick += count_tick_len; // set for next tick 
 	
-	if ( RightPressed () || MouseWheelDownPressed() )
-	{
-	    YourColor = VIOLETT;
-	    OpponentColor = GELB;
-	}
-	if (LeftPressed () || MouseWheelUpPressed() )
-	{
-	    YourColor = GELB;
-	    OpponentColor = VIOLETT;
-	}
-	
-	if ( SpacePressed() || axis_is_active )
-	{
-	    ColorChosen = TRUE;
-	    while ( SpacePressed() || axis_is_active ) ;
+	while (SDL_PollEvent(&event)) {
+
+	    if (event.type == SDL_QUIT) {
+		Terminate(0);
+	    }
+	    
+	    if (event.type == SDL_MOUSEBUTTONDOWN) {
+		switch(event.button.button) {
+		    //(clever?) hack : mouse wheel up and down behave
+		    //exactly like LEFT and RIGHT arrow, so we mangle the event
+		    case SDL_BUTTON_WHEELUP:
+			event.type = SDL_KEYDOWN;
+			event.key.keysym.sym = SDLK_LEFT;
+			break;
+		    case SDL_BUTTON_WHEELDOWN:
+			event.type = SDL_KEYDOWN;
+			event.key.keysym.sym = SDLK_RIGHT;
+			break;
+		    case SDL_BUTTON_LEFT:
+			ColorChosen = TRUE;
+			break;
+
+		    default: break;
+		}
+	    } 
+	    
+	    /* no else there! (mouse wheel) */
+	    if (event.type == SDL_KEYDOWN) { 
+		switch(event.key.keysym.sym) {
+		    case SDLK_RIGHT:
+			YourColor = VIOLETT;
+			OpponentColor = GELB;
+			break;
+		    case SDLK_LEFT:
+			YourColor = GELB;
+			OpponentColor = VIOLETT;
+			break;
+		    case SDLK_SPACE:
+			ColorChosen = TRUE;
+			break;
+		    default:
+			break;
+		}
+	    }
+	    
 	}
 	
 	countdown--; // Count down 
-	sprintf (count_text, "Color-%d", countdown);
+	sprintf (count_text, _("Color-%d"), countdown);
 	
 	ShowPlayground ();
 	to_show_banner (count_text, NULL);
-	our_SDL_flip_wrapper (Screen);
+	our_SDL_flip_wrapper();
 	
 	if (countdown == 0)
 	  ColorChosen = TRUE;
       
     } // while(!ColorChosen) 
-  
-  return;
-  
+
+    while (MouseLeftPressed())
+	SDL_Delay(1);
 }; // void ChooseColor ( void ) 
 
 
@@ -481,7 +506,9 @@ PlayGame (void)
   int up, down, set; 
   int up_counter, down_counter; 
 
-  sprintf (count_text, "Subliminal");   /* Make sure a value gets assigned to count_text */
+  SDL_Event event;
+
+  sprintf (count_text, _("Subliminal"));   /* Make sure a value gets assigned to count_text */
   count_tick_len = 100;   /* countdown in 1/10 second steps */
   move_tick_len  = 60;    /* allow motion at this tick-speed in ms */
   
@@ -495,30 +522,47 @@ PlayGame (void)
   while (!FinishTakeover)
     {
       cur_time = SDL_GetTicks ();
-      
+
       /* 
        * here we register if there have been key-press events in the
        * "waiting period" between move-ticks :
        */
-      up   = ( up   | UpPressed() ) + MouseWheelUpPressed() ; 
-      down = ( down | DownPressed() ) + MouseWheelDownPressed() ;
-      set  = set  | SpacePressed();
+      up   = ( up   | UpPressed() ); 
+      down = ( down | DownPressed() );
+      set  = set  | SpacePressed() | MouseLeftPressed();
+
+      while (SDL_PollEvent(&event)) {
+	  if (event.type == SDL_MOUSEBUTTONDOWN) {
+	      switch (event.button.button) {
+		  case SDL_BUTTON_WHEELUP:
+		      up ++;
+		      break;
+		  case SDL_BUTTON_WHEELDOWN:
+		      down ++;
+		      break;
+		  default:
+		      break;
+	      }
+	  } else if (event.type == SDL_KEYDOWN) {
+	      /* allow for a WIN-key that give immedate victory */
+	      event.key.keysym.mod &= ~(KMOD_CAPS | KMOD_NUM | KMOD_MODE); /* We want to ignore "global" modifiers. */
+	      if (event.key.keysym.sym == SDLK_w && (event.key.keysym.mod == (KMOD_LCTRL | KMOD_LALT))) {
+		  LeaderColor = YourColor;   /* simple as that */
+		  return;  /* leave now, to avoid changing of LeaderColor! */
+	      }
+	  } else if (event.type == SDL_QUIT) {
+	      Terminate(0);
+	  }
+      }
 
       if (!up) up_counter = 0;    /* reset counters for released keys */
       if (!down) down_counter =0;
 
-      /* allow for a WIN-key that give immedate victory */
-      if ( WPressed () && CtrlWasPressed () && Alt_Was_Pressed () )
-	{
-	  LeaderColor = YourColor;   /* simple as that */
-	  return;  /* leave now, to avoid changing of LeaderColor! */
-	} 
-	
       if ( cur_time > prev_count_tick + count_tick_len ) /* time to count 1 down */
 	{
 	  prev_count_tick += count_tick_len;  /* set for next countdown tick */
 	  countdown--;
-	  sprintf (count_text, "Finish-%d", countdown);
+	  sprintf (count_text, _("Finish-%d"), countdown);
 
 	  if (countdown == 0)
 	    FinishTakeover = TRUE;
@@ -587,6 +631,11 @@ PlayGame (void)
 		  CapsuleCountdown[YourColor][0][row] = CAPSULE_COUNTDOWN * 2;
 
 		  Takeover_Set_Capsule_Sound ();
+
+		  if (!NumCapsules[YOU]) {
+		      /* You placed your last capsule ? let's speed up the end */
+		      count_tick_len *= 0.75;
+		  }
 		}	/* if (row > 0 && ... ) */
 	    } /* if ( set ) */
 
@@ -603,7 +652,8 @@ PlayGame (void)
 
       ShowPlayground ();
       to_show_banner (count_text, NULL);
-      our_SDL_flip_wrapper (Screen);
+      our_SDL_flip_wrapper();
+      SDL_Delay(10);
     }	/* while !FinishTakeover */
 
   /* Final contdown */
@@ -623,7 +673,7 @@ PlayGame (void)
       ProcessPlayground ();	/* this has to be done several times to be sure */
       ProcessDisplayColumn ();
       ShowPlayground ();
-      our_SDL_flip_wrapper (Screen);
+      our_SDL_flip_wrapper();
     }	/* while (countdown) */
 
     return;
@@ -809,7 +859,7 @@ AdvancedEnemyTakeoverMovements (void)
   return;
 }; // AdvancedEnemyTakeoverMovements 
 
-/* ----------------------------------------------------------------------
+/**
  * This function reads in the takeover game elements for later blitting. 
  * It frees previous SDL-surfaces if they were allocated.  T
  * This allows to use this fct also for theme-switching.
@@ -834,11 +884,12 @@ GetTakeoverGraphics (void)
   // Now we start loading all the takeover graphics.
   //
   Set_Rect (tmp, User_Rect.x, User_Rect.y, 0, 0);
-
-  TempLoadSurface = our_IMG_load_wrapper ( find_file ( TO_BLOCK_FILE , GRAPHICS_DIR , TRUE ) ) ;
+  char fp[2048];
+  find_file ( TO_BLOCK_FILE , GRAPHICS_DIR , fp, 0);
+  TempLoadSurface = our_IMG_load_wrapper ( fp ) ;
   to_blocks = our_SDL_display_format_wrapperAlpha ( TempLoadSurface ); // the surface is converted
   SDL_FreeSurface ( TempLoadSurface );
-  if ( use_open_gl ) flip_image_horizontally ( to_blocks );
+  if ( use_open_gl ) flip_image_vertically ( to_blocks );
 
   // Get the fill-blocks 
   for ( i = 0 ; i < NUM_FILL_BLOCKS ; i++ , curx += FILL_BLOCK_LEN + 2 )
@@ -940,18 +991,20 @@ ShowPlayground ( void )
     blit_special_background ( TAKEOVER_BACKGROUND_CODE );
     
     blit_tux ( xoffs + DruidStart [ YourColor ] . x ,
-	       yoffs + DruidStart [ YourColor ] . y + 30 , 0 );
+	       yoffs + DruidStart [ YourColor ] . y + 30);
 
-    if ( AllEnemys [ DroidNum ] . Status != INFOUT )
-	PutEnemy (DroidNum, xoffs + DruidStart[!YourColor].x,
+    float saved_phase=cDroid->previous_phase;
+    cDroid->previous_phase=0;
+    PutEnemy (cDroid, xoffs + DruidStart[!YourColor].x,
 		  yoffs + DruidStart[!YourColor].y , FALSE , FALSE );
-    
+
+    cDroid->previous_phase=saved_phase;    
     Set_Rect ( Target_Rect, xoffs + LEFT_OFFS_X, yoffs + LEFT_OFFS_Y,
 	       User_Rect.w, User_Rect.h);
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( ToGroundBlocks [ GELB_OBEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ GELB_OBEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ GELB_OBEN ] . surface , NULL , Screen, &Target_Rect );
     
@@ -960,16 +1013,16 @@ ShowPlayground ( void )
     for ( i = 0 ; i < 12 ; i++ )
     {
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( ToGroundBlocks [ GELB_MITTE ] , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ GELB_MITTE ] , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( ToGroundBlocks [ GELB_MITTE ] . surface , NULL , Screen, &Target_Rect );
 	Target_Rect.y += GROUNDBLOCKHEIGHT;
     } 
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( ToGroundBlocks [ GELB_UNTEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ GELB_UNTEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ GELB_UNTEN ] . surface , NULL , Screen, &Target_Rect );
     
@@ -977,8 +1030,8 @@ ShowPlayground ( void )
     Set_Rect (Target_Rect, xoffs + MID_OFFS_X, yoffs + MID_OFFS_Y,0, 0);
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( ToLeaderBlock , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToLeaderBlock , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToLeaderBlock . surface , NULL , Screen, &Target_Rect);
     
@@ -986,8 +1039,8 @@ ShowPlayground ( void )
     for ( i = 0 ; i < 12 ; i++ , Target_Rect.y += COLUMNBLOCKHEIGHT )
     {
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( ToColumnBlock , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &ToColumnBlock , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( ToColumnBlock . surface , NULL , Screen, &Target_Rect );
     }
@@ -996,8 +1049,8 @@ ShowPlayground ( void )
     // the right column
     Set_Rect ( Target_Rect , xoffs + RIGHT_OFFS_X , yoffs + RIGHT_OFFS_Y , 0 , 0 );
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( ToGroundBlocks [ VIOLETT_OBEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ VIOLETT_OBEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ VIOLETT_OBEN ] . surface , NULL , Screen , &Target_Rect );
     
@@ -1006,30 +1059,30 @@ ShowPlayground ( void )
     for ( i = 0 ; i < 12 ; i++ , Target_Rect.y += GROUNDBLOCKHEIGHT )
     {
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( ToGroundBlocks [ VIOLETT_MITTE ] , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ VIOLETT_MITTE ] , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( ToGroundBlocks [ VIOLETT_MITTE ] . surface , NULL , Screen, &Target_Rect );
     }
     
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( ToGroundBlocks [ VIOLETT_UNTEN ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &ToGroundBlocks [ VIOLETT_UNTEN ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( ToGroundBlocks [ VIOLETT_UNTEN ] . surface , NULL , Screen, &Target_Rect );
     
     // Fill the leader-LED with its color 
     Set_Rect (Target_Rect, xoffs + LEADERLED_X, yoffs + LEADERLED_Y, 0, 0);
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( FillBlocks [ LeaderColor ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &FillBlocks [ LeaderColor ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( FillBlocks [ LeaderColor ] . surface , NULL , Screen, &Target_Rect );
     
     Target_Rect.y += FILL_BLOCK_HEIGHT;
     if ( use_open_gl ) 
-	blit_open_gl_texture_to_screen_position ( FillBlocks [ LeaderColor ] , Target_Rect . x ,
-						  Target_Rect . y , TRUE ) ;
+	draw_gl_textured_quad_at_screen_position ( &FillBlocks [ LeaderColor ] , Target_Rect . x ,
+						  Target_Rect . y ) ;
     else
 	our_SDL_blit_surface_wrapper ( FillBlocks [ LeaderColor ] . surface , NULL , Screen, &Target_Rect );
     
@@ -1040,8 +1093,8 @@ ShowPlayground ( void )
 		  yoffs + LEDCOLUMN_Y + i*(FILL_BLOCK_HEIGHT+2),
 		0, 0);
 	if ( use_open_gl ) 
-	    blit_open_gl_texture_to_screen_position ( FillBlocks [ DisplayColumn [ i ] ] , Target_Rect . x ,
-						      Target_Rect . y , TRUE ) ;
+	    draw_gl_textured_quad_at_screen_position ( &FillBlocks [ DisplayColumn [ i ] ] , Target_Rect . x ,
+						      Target_Rect . y ) ;
 	else
 	    our_SDL_blit_surface_wrapper ( FillBlocks [ DisplayColumn [ i ] ] . surface , NULL , Screen, &Target_Rect );
     }
@@ -1055,8 +1108,8 @@ ShowPlayground ( void )
 		      yoffs + PlaygroundStart[GELB].y + j * TO_BLOCKHEIGHT, 0, 0);
 	    block = ToPlayground[GELB][i][j] + ActivationMap[GELB][i][j]*TO_BLOCKS;
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( ToGameBlocks [ block ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &ToGameBlocks [ block ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( ToGameBlocks [ block ] . surface , NULL , Screen, &Target_Rect );
 	}
@@ -1072,8 +1125,8 @@ ShowPlayground ( void )
 	    block = ToPlayground[VIOLETT][i][j]+
 		(NUM_PHASES+ActivationMap[VIOLETT][i][j])*TO_BLOCKS;
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( ToGameBlocks [ block ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &ToGameBlocks [ block ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( ToGameBlocks [ block ] . surface , NULL , Screen, &Target_Rect );
 	}
@@ -1092,8 +1145,8 @@ ShowPlayground ( void )
 	if ( NumCapsules [ player ] )
 	{
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( CapsuleBlocks [ color ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &CapsuleBlocks [ color ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( CapsuleBlocks [ color ] . surface , NULL , Screen, &Target_Rect );
 	}
@@ -1103,8 +1156,8 @@ ShowPlayground ( void )
 	    Set_Rect ( Target_Rect , xoffs + LeftCapsulesStart [ color ] . x ,
 		       yoffs + LeftCapsulesStart [ color ] . y + i * CAPSULE_HEIGHT , 0 , 0 );
 	    if ( use_open_gl ) 
-		blit_open_gl_texture_to_screen_position ( CapsuleBlocks [ color ] , Target_Rect . x ,
-							  Target_Rect . y , TRUE ) ;
+		draw_gl_textured_quad_at_screen_position ( &CapsuleBlocks [ color ] , Target_Rect . x ,
+							  Target_Rect . y ) ;
 	    else
 		our_SDL_blit_surface_wrapper ( CapsuleBlocks [ color ] . surface , NULL , Screen, &Target_Rect );
 	} // for capsules 
@@ -1580,11 +1633,11 @@ ProcessPlayground (void)
   return;
 };  // void ProcessPlayground ( void )
 
-/* ---------------------------------------------------------------------- 
+/** 
  * This function sets the correct values for the status column in the
  * middle of the takeover game field.
  * Binking leds are realized here as well.
- * ---------------------------------------------------------------------- */
+ */
 void
 ProcessDisplayColumn (void)
 {
@@ -1666,15 +1719,15 @@ ProcessDisplayColumn (void)
   // depend on the details of the final takeover score.  Therefore we set this
   // resistance factor variable here.
   //
-  Me[0].Current_Victim_Resistance_Factor = 0.2 * ( (float) 12 - abs( ViolettCounter- GelbCounter ) );
+  Me.Current_Victim_Resistance_Factor = 0.2 * ( (float) 12 - abs( ViolettCounter- GelbCounter ) );
 
   return;
 }; // void ProcessDisplayColumn 
 
-/* ---------------------------------------------------------------------- 
+/** 
  * This function does the countdown of the capsules and kills them if 
  * they are too old.
- * ---------------------------------------------------------------------- */
+ */
 void
 ProcessCapsules (void)
 {
@@ -1698,10 +1751,10 @@ ProcessCapsules (void)
 
 }; // void ProcessCapsules ( void )
 
-/* ----------------------------------------------------------------------
+/**
  * This function tells, wether a Column-connection is active or not.
  * It returns TRUE or FALSE accordinly.
- * ---------------------------------------------------------------------- */
+ */
 int
 IsActive (int color, int row)
 {
@@ -1738,10 +1791,10 @@ AnimateCurrents (void)
   return;
 }; // void AnimateCurrents (void)
 
-/* ----------------------------------------------------------------------
+/**
  *
  *
- * ---------------------------------------------------------------------- */
+ */
 void
 to_show_banner (const char* left, const char* right)
 {
@@ -1760,7 +1813,7 @@ to_show_banner (const char* left, const char* right)
 
   if ( right == NULL )
     {
-      sprintf ( dummy , "Score" );
+      sprintf ( dummy , _("Score"));
       right = dummy;
     }
 
